@@ -44,6 +44,7 @@ public:
     DPXInput () : m_stream(NULL), m_dataPtr(NULL) { init(); }
     virtual ~DPXInput () { close(); }
     virtual const char * format_name (void) const { return "dpx"; }
+    virtual bool valid_file (const std::string &filename) const;
     virtual bool open (const std::string &name, ImageSpec &newspec);
     virtual bool close ();
     virtual int current_subimage (void) const { return m_subimage; }
@@ -85,11 +86,11 @@ private:
 // Obligatory material to make this a recognizeable imageio plugin:
 OIIO_PLUGIN_EXPORTS_BEGIN
 
-DLLEXPORT ImageInput *dpx_input_imageio_create () { return new DPXInput; }
+OIIO_EXPORT ImageInput *dpx_input_imageio_create () { return new DPXInput; }
 
-DLLEXPORT int dpx_imageio_version = OIIO_PLUGIN_VERSION;
+OIIO_EXPORT int dpx_imageio_version = OIIO_PLUGIN_VERSION;
 
-DLLEXPORT const char * dpx_input_extensions[] = {
+OIIO_EXPORT const char * dpx_input_extensions[] = {
     "dpx", NULL
 };
 
@@ -97,6 +98,25 @@ OIIO_PLUGIN_EXPORTS_END
 
 
 
+bool
+DPXInput::valid_file (const std::string &filename) const
+{
+    InStream *stream = new InStream();
+    if (! stream)
+        return false;
+    bool ok = false;
+    if (stream->Open(filename.c_str())) {
+        dpx::Reader dpx;
+        dpx.SetInStream(stream);
+        ok = dpx.ReadHeader();
+        stream->Close();
+    }
+    delete stream;
+    return ok;
+}
+
+
+
 bool
 DPXInput::open (const std::string &name, ImageSpec &newspec)
 {
