@@ -29,6 +29,7 @@
 */
 
 #include "imageviewer.h"
+#include "ivgl.h"
 
 #include <iostream>
 #include <cmath>
@@ -40,12 +41,29 @@
 #include <boost/foreach.hpp>
 #include <boost/filesystem.hpp>
 
+#include <QtCore/QSettings>
+#include <QtCore/QTimer>
+#include <QtGui/QApplication>
+#include <QtGui/QComboBox>
+#include <QtGui/QDesktopWidget>
+#include <QtGui/QFileDialog>
+#include <QtGui/QKeyEvent>
+#include <QtGui/QLabel>
+#include <QtGui/QMenu>
+#include <QtGui/QMenuBar>
+#include <QtGui/QMessageBox>
+#include <QtGui/QProgressBar>
+#include <QtGui/QResizeEvent>
+#include <QtGui/QSpinBox>
+#include <QtGui/QStatusBar>
+
 #include <OpenEXR/ImathFun.h>
 
 #include "dassert.h"
 #include "strutil.h"
 #include "timer.h"
 #include "fmath.h"
+#include "ivutils.h"
 #include "sysutil.h"
 #include "filesystem.h"
 
@@ -58,6 +76,38 @@ namespace
 
 }
 
+
+
+static const char *s_file_filters = ""
+    "Image Files (*.bmp *.cin *.dds *.dpx *.f3d *.fits *.hdr *.ico *.iff *.jpg "
+    "*.jpe *.jpeg *.jif *.jfif *.jfi *.jp2 *.j2k *.exr *.png *.pbm *.pgm *.ppm "
+    "*.ptex *.rla *.sgi *.rgb *.rgba *.bw *.int *.inta *.pic *.tga *.tpic "
+    "*.tif *.tiff *.tx *.env *.sm *.vsm *.zfile);;"
+    "BMP (*.bmp);;"
+    "Cineon (*.cin);;"
+    "Direct Draw Surface (*.dds);;"
+    "DPX (*.dpx);;"
+    "Field3D (*.f3d);;"
+    "FITS (*.fits);;"
+    "HDR/RGBE (*.hdr);;"
+    "Icon (*.ico);;"
+    "IFF (*.iff);;"
+    "JPEG (*.jpg *.jpe *.jpeg *.jif *.jfif *.jfi);;"
+    "JPEG-2000 (*.jp2 *.j2k);;"
+    "OpenEXR (*.exr);;"
+    "Portable Network Graphics (*.png);;"
+    "PNM / Netpbm (*.pbm *.pgm *.ppm);;"
+    "Ptex (*.ptex);;"
+    "RLA (*.rla);;"
+    "SGI (*.sgi *.rgb *.rgba *.bw *.int *.inta);;"
+    "Softimage PIC (*.pic);;"
+    "Targa (*.tga *.tpic);;"
+    "TIFF (*.tif *.tiff *.tx *.env *.sm *.vsm);;"
+    "Zfile (*.zfile);;"
+    "All Files (*)";
+
+
+
 ImageViewer::ImageViewer ()
     : infoWindow(NULL), preferenceWindow(NULL), darkPaletteBox(NULL),
       m_current_image(-1), m_current_channel(0), m_color_mode(RGBA),
@@ -399,12 +449,10 @@ ImageViewer::createMenus()
     fileMenu->addAction (exitAct);
     menuBar()->addMenu (fileMenu);
 
-    editMenu = new QMenu(tr("&Edit"), this);
     // Copy
     // Paste
     // Clear selection
     // radio: prioritize selection, crop selection
-    menuBar()->addMenu (editMenu);
 
     expgamMenu = new QMenu(tr("Exposure/gamma"));  // submenu
     expgamMenu->addAction (exposureMinusOneHalfStopAct);
@@ -599,14 +647,18 @@ image_progress_callback (void *opaque, float done)
 void
 ImageViewer::open()
 {
-    QStringList names;
-    names = QFileDialog::getOpenFileNames (this, tr("Open File(s)"),
-                                           QDir::currentPath());
-    if (names.empty())
+    static QString openPath = QDir::currentPath();
+    QFileDialog dialog(NULL, tr("Open File(s)"),
+                       openPath, tr(s_file_filters));
+    dialog.setAcceptMode (QFileDialog::AcceptOpen);
+    dialog.setFileMode (QFileDialog::ExistingFiles);
+    if (!dialog.exec())
         return;
+    openPath = dialog.directory().path();
+    QStringList names = dialog.selectedFiles();
+
     int old_lastimage = m_images.size()-1;
-    QStringList list = names;
-    for (QStringList::Iterator it = list.begin();  it != list.end();  ++it) {
+    for (QStringList::Iterator it = names.begin();  it != names.end();  ++it) {
         std::string filename = it->toUtf8().data();
         if (filename.empty())
             continue;
@@ -746,7 +798,8 @@ ImageViewer::saveAs()
         return;
     QString name;
     name = QFileDialog::getSaveFileName (this, tr("Save Image"),
-                                         QString(img->name().c_str()));
+                                         QString(img->name().c_str()),
+                                         tr(s_file_filters));
     if (name.isEmpty())
         return;
     bool ok = img->save (name.toStdString(), "", image_progress_callback, this);
@@ -1680,12 +1733,20 @@ ImageViewer::closeImg()
     m_images[m_current_image] = NULL;
     m_images.erase (m_images.begin()+m_current_image);
 
-    // FIXME:
-    // For all image indices we may be storing,
-    //   if == m_current_image, wrap to 0 if this was the last image
-    //   else if > m_current_image, subtract one
+    // Update image indices
+    // This should be done for all image indices we may be storing
+    if (m_last_image == m_current_image)
+    {
+        if (!m_images.empty() && m_last_image > 0)
+            m_last_image = 0;
+        else
+            m_last_image = -1;
+    }
+    if (m_last_image > m_current_image)
+        m_last_image --;
 
-    current_image (current_image() < (int)m_images.size() ? current_image() : 0);
+    m_current_image = m_current_image < (int)m_images.size() ? m_current_image : 0;
+    displayCurrentImage ();
 }
 
 
@@ -1718,13 +1779,8 @@ void ImageViewer::zoomIn()
     if (zoom() >= 64)
         return;
     float oldzoom = zoom ();
-    float newzoom;
-    if (zoom() >= 1.0f) {
-        newzoom = pow2roundup ((int) round (zoom()) + 1);
-    } else {
-        newzoom = 1.0 / std::max (1, pow2rounddown ((int) round(1.0/zoom()) - 1));
-    }
-
+    float newzoom = pow2roundupf (oldzoom);
+    
     float xc, yc;  // Center view position
     glwin->get_center (xc, yc);
     int xm, ym;  // Mouse position
@@ -1755,16 +1811,8 @@ void ImageViewer::zoomOut()
     if (zoom() <= 1.0f/64)
         return;
     float oldzoom = zoom ();
-    float newzoom;
-    if (zoom() > 1.0f) {
-        int z = pow2rounddown ((int) zoom() - 1);
-        newzoom = std::max ((float)z, 0.5f);
-    } else {
-        int z = (int)(1.0 / zoom() + 0.001);  // add for floating point slop
-        z = pow2roundup (z+1);
-        newzoom = 1.0f / z;
-    }
-
+    float newzoom = pow2rounddownf (oldzoom);
+    
     float xcpel, ycpel;  // Center view position
     glwin->get_center (xcpel, ycpel);
     int xmpel, ympel;  // Mouse position
@@ -1938,7 +1986,7 @@ ImageViewer::about()
     QMessageBox::about(this, tr("About iv"),
             tr("<p><b>iv</b> is the image viewer for OpenImageIO.</p>"
                "<p>(c) Copyright 2008 Larry Gritz et al.  All Rights Reserved.</p>"
-               "<p>See http://openimageio.org for details.</p>"));
+               "<p>See <a href='http://openimageio.org'>http://openimageio.org</a> for details.</p>"));
 }
 
 
