@@ -46,6 +46,52 @@ OIIO_NAMESPACE_ENTER
     using namespace pvt;
 
 
+
+// Default implementation of valid_file: try to do a full open.  If it
+// succeeds, it's the right kind of file.  We assume that most plugins
+// will override this with something smarter and much less expensive,
+// like reading just the first few bytes of the file to check for magic
+// numbers.
+bool
+ImageInput::valid_file (const std::string &filename) const
+{
+    ImageSpec tmpspec;
+    bool ok = const_cast<ImageInput *>(this)->open (filename, tmpspec);
+    if (ok)
+        const_cast<ImageInput *>(this)->close ();
+    return ok;
+}
+
+
+
+ImageInput *
+ImageInput::open (const std::string &filename,
+                  const ImageSpec *config)
+{
+    if (config == NULL) {
+        // Without config, this is really just a call to create-with-open.
+        return ImageInput::create (filename, true, std::string());
+    }
+
+    // With config, create without open, then try to open with config.
+    ImageInput *in = ImageInput::create (filename, false, std::string());
+    if (! in)
+        return NULL;  // create() failed
+    ImageSpec newspec;
+    if (in->open (filename, newspec, *config))
+        return in;   // creted fine, opened fine, return it
+
+    // The open failed.  Transfer the error from 'in' to the global OIIO
+    // error, delete the ImageInput we allocated, and return NULL.
+    std::string err = in->geterror();
+    if (err.size())
+        pvt::error ("%s", err.c_str());
+    delete in;
+    return NULL;
+}
+
+
+
 bool 
 ImageInput::read_scanline (int y, int z, TypeDesc format, void *data,
                            stride_t xstride)
@@ -86,13 +132,9 @@ ImageInput::read_scanline (int y, int z, TypeDesc format, void *data,
                              data, format, xstride, AutoStride, AutoStride);
     } else {
         // Per-channel formats -- have to convert/copy channels individually
-        if (native_data) {
-            ASSERT (contiguous && "Per-channel native input requires contiguous strides");
-        }
-        ASSERT (format != TypeDesc::UNKNOWN);
         ASSERT (m_spec.channelformats.size() == (size_t)m_spec.nchannels);
         size_t offset = 0;
-        for (int c = 0;  c < m_spec.nchannels;  ++c) {
+        for (int c = 0;  ok && c < m_spec.nchannels;  ++c) {
             TypeDesc chanformat = m_spec.channelformats[c];
             ok = convert_image (1 /* channels */, m_spec.width, 1, 1, 
                                 buf+offset, chanformat, 
@@ -124,13 +166,14 @@ ImageInput::read_scanlines (int ybegin, int yend, int z,
 
 bool
 ImageInput::read_scanlines (int ybegin, int yend, int z,
-                            int firstchan, int nchans,
+                            int chbegin, int chend,
                             TypeDesc format, void *data,
                             stride_t xstride, stride_t ystride)
 {
-    nchans = std::min (nchans, m_spec.nchannels-firstchan);
+    chend = clamp (chend, chbegin+1, m_spec.nchannels);
+    int nchans = chend - chbegin;
     yend = std::min (yend, spec().y+spec().height);
-    size_t native_pixel_bytes = m_spec.pixel_bytes (firstchan, nchans, true);
+    size_t native_pixel_bytes = m_spec.pixel_bytes (chbegin, chend, true);
     imagesize_t native_scanline_bytes = clamped_mult64 ((imagesize_t)m_spec.width,
                                                         (imagesize_t)native_pixel_bytes);
     bool native = (format == TypeDesc::UNKNOWN);
@@ -147,11 +190,10 @@ ImageInput::read_scanlines (int ybegin, int yend, int z,
     bool rightformat = (format == TypeDesc::UNKNOWN) ||
         (format == m_spec.format && m_spec.channelformats.empty());
     if (rightformat && contiguous) {
-        if (firstchan == 0 && nchans == m_spec.nchannels)
+        if (chbegin == 0 && chend == m_spec.nchannels)
             return read_native_scanlines (ybegin, yend, z, data);
         else
-            return read_native_scanlines (ybegin, yend, z,
-                                          firstchan, nchans, data);
+            return read_native_scanlines (ybegin, yend, z, chbegin, chend, data);
     }
 
     // No such luck.  Read scanlines in chunks.
@@ -164,7 +206,7 @@ ImageInput::read_scanlines (int ybegin, int yend, int z,
     int scanline_values = m_spec.width * nchans;
     for (;  ok && ybegin < yend;  ybegin += chunk) {
         int y1 = std::min (ybegin+chunk, yend);
-        ok &= read_native_scanlines (ybegin, y1, z, firstchan, nchans, &buf[0]);
+        ok &= read_native_scanlines (ybegin, y1, z, chbegin, chend, &buf[0]);
         if (! ok)
             break;
 
@@ -183,11 +225,11 @@ ImageInput::read_scanlines (int ybegin, int yend, int z,
             // Per-channel formats -- have to convert/copy channels individually
             size_t offset = 0;
             for (int c = 0;  ok && c < nchans;  ++c) {
-                TypeDesc chanformat = m_spec.channelformats[c+firstchan];
+                TypeDesc chanformat = m_spec.channelformats[c+chbegin];
                 ok = convert_image (1 /* channels */, m_spec.width, nscanlines, 1, 
                                     &buf[offset], chanformat, 
-                                    pixel_bytes, AutoStride, AutoStride,
-                                    (char *)data + c*m_spec.format.size(),
+                                    native_pixel_bytes, AutoStride, AutoStride,
+                                    (char *)data + c*format.size(),
                                     format, xstride, ystride, zstride);
                 offset += chanformat.size ();
             }
@@ -224,17 +266,17 @@ ImageInput::read_native_scanlines (int ybegin, int yend, int z, void *data)
 
 bool
 ImageInput::read_native_scanlines (int ybegin, int yend, int z,
-                                   int firstchan, int nchans, void *data)
+                                   int chbegin, int chend, void *data)
 {
     // All-channel case just reduces to the simpler read_native_scanlines.
-    if (firstchan == 0 && nchans >= m_spec.nchannels)
+    if (chbegin == 0 && chend >= m_spec.nchannels)
         return read_native_scanlines (ybegin, yend, z, data);
 
     // Base class implementation of read_native_scanlines (with channel
     // subset) just calls read_native_scanlines (all channels), and
     // copies the appropriate subset.
-    size_t prefix_bytes = m_spec.pixel_bytes (0,firstchan,true);
-    size_t subset_bytes = m_spec.pixel_bytes (firstchan,nchans,true);
+    size_t prefix_bytes = m_spec.pixel_bytes (0,chbegin,true);
+    size_t subset_bytes = m_spec.pixel_bytes (chbegin,chend,true);
     size_t subset_ystride = m_spec.width * subset_bytes;
 
     size_t native_pixel_bytes = m_spec.pixel_bytes (true);
@@ -304,10 +346,6 @@ ImageInput::read_tile (int x, int y, int z, TypeDesc format, void *data,
                              data, format, xstride, ystride, zstride);
     } else {
         // Per-channel formats -- have to convert/copy channels individually
-        if (native_data) {
-            ASSERT (contiguous && "Per-channel native input requires contiguous strides");
-        }
-        ASSERT (format != TypeDesc::UNKNOWN);
         ASSERT (m_spec.channelformats.size() == (size_t)m_spec.nchannels);
         size_t offset = 0;
         for (int c = 0;  c < m_spec.nchannels;  ++c) {
@@ -346,17 +384,18 @@ ImageInput::read_tiles (int xbegin, int xend, int ybegin, int yend,
 bool 
 ImageInput::read_tiles (int xbegin, int xend, int ybegin, int yend,
                         int zbegin, int zend, 
-                        int firstchan, int nchans,
+                        int chbegin, int chend,
                         TypeDesc format, void *data,
                         stride_t xstride, stride_t ystride, stride_t zstride)
 {
     if (! m_spec.valid_tile_range (xbegin, xend, ybegin, yend, zbegin, zend))
         return false;
 
-    nchans = std::min (nchans, m_spec.nchannels-firstchan);
+    chend = clamp (chend, chbegin+1, m_spec.nchannels);
+    int nchans = chend - chbegin;
     // native_pixel_bytes is the size of a pixel in the FILE, including
     // the per-channel format.
-    stride_t native_pixel_bytes = (stride_t) m_spec.pixel_bytes (firstchan, nchans, true);
+    stride_t native_pixel_bytes = (stride_t) m_spec.pixel_bytes (chbegin, chend, true);
     // perchanfile is true if the file has different per-channel formats
     bool perchanfile = m_spec.channelformats.size();
     // native_data is true if the user asking for data in the native format
@@ -384,12 +423,12 @@ ImageInput::read_tiles (int xbegin, int xend, int ybegin, int yend,
         (xend-xbegin) == nxtiles*m_spec.tile_width &&
         (yend-ybegin) == nytiles*m_spec.tile_height &&
         (zend-zbegin) == nztiles*m_spec.tile_depth) {
-        if (firstchan == 0 && nchans == m_spec.nchannels)
+        if (chbegin == 0 && chend == m_spec.nchannels)
             return read_native_tiles (xbegin, xend, ybegin, yend, zbegin, zend,
                                       data);  // Simple case
         else
             return read_native_tiles (xbegin, xend, ybegin, yend, zbegin, zend,
-                                      firstchan, nchans, data);
+                                      chbegin, chend, data);
     }
 
     // No such luck.  Just punt and read tiles individually.
@@ -398,7 +437,10 @@ ImageInput::read_tiles (int xbegin, int xend, int ybegin, int yend,
                                      : (format.size() * nchans);
     stride_t full_pixelsize = native_data ? m_spec.pixel_bytes(true)
                                           : (format.size() * m_spec.nchannels);
-    size_t prefix_bytes = m_spec.pixel_bytes (0,firstchan,true);
+    stride_t full_tilewidthbytes = full_pixelsize * m_spec.tile_width;
+    stride_t full_tilewhbytes = full_tilewidthbytes * m_spec.tile_height;
+    stride_t full_tilebytes = full_tilewhbytes * m_spec.tile_depth;
+    size_t prefix_bytes = m_spec.pixel_bytes (0,chbegin,true);
     std::vector<char> buf;
     for (int z = zbegin;  z < zend;  z += std::max(1,m_spec.tile_depth)) {
         int zd = std::min (zend-z, m_spec.tile_depth);
@@ -413,25 +455,34 @@ ImageInput::read_tiles (int xbegin, int xend, int ybegin, int yend,
                 // partial channel subsets are read into a buffer and
                 // then copied.
                 if (xw == m_spec.tile_width && yh == m_spec.tile_height &&
-                      zd == m_spec.tile_depth &&
-                      firstchan == 0 && nchans == m_spec.nchannels) {
+                      zd == m_spec.tile_depth && !perchanfile &&
+                      chbegin == 0 && chend == m_spec.nchannels) {
+                    // Full tile, either native data or not needing
+                    // per-tile data format conversion.
                     ok &= read_tile (x, y, z, format, tilestart,
                                      xstride, ystride, zstride);
                 } else {
-                    buf.resize (m_spec.tile_bytes());
-                    ok &= read_tile (x, y, z, format, &buf[0],
-                                     full_pixelsize,
-                                     full_pixelsize*m_spec.tile_width,
-                                     full_pixelsize*m_spec.tile_pixels());
+                    buf.resize (full_tilebytes);
+                    ok &= read_tile (x, y, z, 
+                                     perchanfile ? TypeDesc::UNKNOWN : format,
+                                     &buf[0], full_pixelsize,
+                                     full_tilewidthbytes, full_tilewhbytes);
                     if (ok)
                         copy_image (nchans, xw, yh, zd, &buf[prefix_bytes],
                                     pixelsize, full_pixelsize,
-                                    full_pixelsize*m_spec.tile_width,
-                                    full_pixelsize*m_spec.tile_pixels(),
+                                    full_tilewidthbytes, full_tilewhbytes,
                                     tilestart, xstride, ystride, zstride);
+                    // N.B. It looks like read_tiles doesn't handle the
+                    // per-channel data types case fully, but it does!
+                    // The call to read_tile() above handles the case of
+                    // per-channel data types, converting to to desired
+                    // format, so all we have to do on our own is the
+                    // copy_image.
                 }
                 tilestart += m_spec.tile_width * xstride;
             }
+            if (! ok)
+                break;
         }
     }
 
@@ -484,10 +535,13 @@ ImageInput::read_native_tiles (int xbegin, int xend, int ybegin, int yend,
 bool
 ImageInput::read_native_tiles (int xbegin, int xend, int ybegin, int yend,
                                int zbegin, int zend, 
-                               int firstchan, int nchans, void *data)
+                               int chbegin, int chend, void *data)
 {
+    chend = clamp (chend, chbegin+1, m_spec.nchannels);
+    int nchans = chend - chbegin;
+
     // All-channel case just reduces to the simpler read_native_scanlines.
-    if (firstchan == 0 && nchans >= m_spec.nchannels)
+    if (chbegin == 0 && chend >= m_spec.nchannels)
         return read_native_tiles (xbegin, xend, ybegin, yend,
                                   zbegin, zend, data);
 
@@ -503,8 +557,8 @@ ImageInput::read_native_tiles (int xbegin, int xend, int ybegin, int yend,
     stride_t native_tileystride = native_pixel_bytes * m_spec.tile_width;
     stride_t native_tilezstride = native_tileystride * m_spec.tile_height;
 
-    size_t prefix_bytes = m_spec.pixel_bytes (0,firstchan,true);
-    size_t subset_bytes = m_spec.pixel_bytes (firstchan,nchans,true);
+    size_t prefix_bytes = m_spec.pixel_bytes (0,chbegin,true);
+    size_t subset_bytes = m_spec.pixel_bytes (chbegin,chend,true);
     stride_t subset_ystride = (xend-xbegin) * subset_bytes;
     stride_t subset_zstride = (yend-ybegin) * subset_ystride;
 
@@ -552,7 +606,7 @@ ImageInput::read_image (TypeDesc format, void *data,
     if (m_spec.tile_width) {
         // Tiled image
         for (int z = 0;  z < m_spec.depth;  z += m_spec.tile_depth) {
-            for (int y = 0;  y < m_spec.height;  y += m_spec.tile_height) {
+            for (int y = 0;  y < m_spec.height && ok;  y += m_spec.tile_height) {
                 ok &= read_tiles (m_spec.x, m_spec.x+m_spec.width,
                                   y+m_spec.y, std::min (y+m_spec.y+m_spec.tile_height, m_spec.y+m_spec.height),
                                   z+m_spec.z, std::min (z+m_spec.z+m_spec.tile_depth, m_spec.z+m_spec.depth),
@@ -584,6 +638,53 @@ ImageInput::read_image (TypeDesc format, void *data,
 
 
 
+bool
+ImageInput::read_native_deep_scanlines (int ybegin, int yend, int z,
+                                        int chbegin, int chend,
+                                        DeepData &deepdata)
+{
+    return false;  // default: doesn't support deep images
+}
+
+
+
+bool
+ImageInput::read_native_deep_tiles (int xbegin, int xend,
+                                    int ybegin, int yend,
+                                    int zbegin, int zend,
+                                    int chbegin, int chend,
+                                    DeepData &deepdata)
+{
+    return false;  // default: doesn't support deep images
+}
+
+
+
+bool
+ImageInput::read_native_deep_image (DeepData &deepdata)
+{
+    if (m_spec.depth > 1) {
+        error ("read_native_deep_image is not supported for volume (3D) images.");
+        return false;
+        // FIXME? - not implementing 3D deep images for now.  The only
+        // format that supports deep images at this time is OpenEXR, and
+        // it doesn't support volumes.
+    }
+    if (m_spec.tile_width) {
+        // Tiled image
+        return read_native_deep_tiles (m_spec.x, m_spec.x+m_spec.width,
+                                       m_spec.y, m_spec.y+m_spec.height,
+                                       m_spec.z, m_spec.z+m_spec.depth,
+                                       0, m_spec.nchannels, deepdata);
+    } else {
+        // Scanline image
+        return read_native_deep_scanlines (m_spec.y, m_spec.y+m_spec.height, 0,
+                                           0, m_spec.nchannels, deepdata);
+    }
+}
+
+
+
 int 
 ImageInput::send_to_input (const char *format, ...)
 {
@@ -603,16 +704,13 @@ ImageInput::send_to_client (const char *format, ...)
 
 
 void 
-ImageInput::error (const char *format, ...)
+ImageInput::append_error (const std::string& message) const
 {
-    va_list ap;
-    va_start (ap, format);
     ASSERT (m_errmessage.size() < 1024*1024*16 &&
             "Accumulated error messages > 16MB. Try checking return codes!");
     if (m_errmessage.size())
         m_errmessage += '\n';
-    m_errmessage += Strutil::vformat (format, ap);
-    va_end (ap);
+    m_errmessage += message;
 }
 
 bool
