@@ -46,15 +46,42 @@
 
 OIIO_NAMESPACE_ENTER
 {
-    using namespace pvt;
+
+// Global private data
+namespace pvt {
+recursive_mutex imageio_mutex;
+int oiio_threads = boost::thread::hardware_concurrency();
+ustring plugin_searchpath;
+std::string format_list;   // comma-separated list of all formats
+std::string extension_list;   // list of all extensions for all formats
+};
+
+
+
+using namespace pvt;
 
 namespace {
 
-static std::string create_error_msg;
 
+// To avoid thread oddities, we have the storage area buffering error
+// messages for seterror()/geterror() be thread-specific.
+static thread_specific_ptr<std::string> thread_error_msg;
+
+// Return a reference to the string for this thread's error messages,
+// creating it if none exists for this thread thus far.
+static std::string &
+error_msg ()
+{
+    std::string *e = thread_error_msg.get();
+    if (! e) {
+        e = new std::string;
+        thread_error_msg.reset (e);
+    }
+    return *e;
 }
 
-recursive_mutex pvt::imageio_mutex;
+} // end anon namespace
+
 
 
 
@@ -69,13 +96,10 @@ openimageio_version ()
 /// Error reporting for the plugin implementation: call this with
 /// printf-like arguments.
 void
-pvt::error (const char *message, ...)
+pvt::seterror (const std::string& message)
 {
     recursive_lock_guard lock (pvt::imageio_mutex);
-    va_list ap;
-    va_start (ap, message);
-    create_error_msg = Strutil::vformat (message, ap);
-    va_end (ap);
+    error_msg() = message;
 }
 
 
@@ -84,8 +108,8 @@ std::string
 geterror ()
 {
     recursive_lock_guard lock (pvt::imageio_mutex);
-    std::string e = create_error_msg;
-    create_error_msg.clear ();
+    std::string e = error_msg();
+    error_msg().clear ();
     return e;
 }
 
@@ -96,7 +120,6 @@ namespace {
 // Private global OIIO data.
 
 static spin_mutex attrib_mutex;
-static int oiio_threads = 1;
 static const int maxthreads = 64;   // reasonable maximum for sanity check
 
 };
@@ -110,14 +133,14 @@ attribute (const std::string &name, TypeDesc type, const void *val)
     if (name == "threads" && type == TypeDesc::TypeInt) {
         oiio_threads = Imath::clamp (*(const int *)val, 0, maxthreads);
         if (oiio_threads == 0) {
-#if (BOOST_VERSION >= 103500)
             oiio_threads = boost::thread::hardware_concurrency();
-#else
-            oiio_threads = 1;
-#endif
         }
         return true;
     }
+    if (name == "plugin_searchpath" && type == TypeDesc::TypeString) {
+        plugin_searchpath = ustring (*(const char **)val);
+        return true;
+    }
     return false;
 }
 
@@ -131,6 +154,22 @@ getattribute (const std::string &name, TypeDesc type, void *val)
         *(int *)val = oiio_threads;
         return true;
     }
+    if (name == "plugin_searchpath" && type == TypeDesc::TypeString) {
+        *(ustring *)val = plugin_searchpath;
+        return true;
+    }
+    if (name == "format_list" && type == TypeDesc::TypeString) {
+        if (format_list.empty())
+            pvt::catalog_all_plugins (plugin_searchpath.string());
+        *(ustring *)val = ustring(format_list);
+        return true;
+    }
+    if (name == "extension_list" && type == TypeDesc::TypeString) {
+        if (extension_list.empty())
+            pvt::catalog_all_plugins (plugin_searchpath.string());
+        *(ustring *)val = ustring(extension_list);
+        return true;
+    }
     return false;
 }
 
@@ -139,7 +178,7 @@ inline int
 quantize (float value, float quant_black, float quant_white,
           int quant_min, int quant_max)
 {
-    value = Imath::lerp (quant_black, quant_white, value);
+    value = Imath::lerp ((float)quant_black, (float)quant_white, value);
     return Imath::clamp ((int)(value + 0.5f), quant_min, quant_max);
 }
 
@@ -512,5 +551,70 @@ copy_image (int nchannels, int width, int height, int depth,
     return true;
 }
 
+
+
+void
+DeepData::init (int npix, int nchan,
+                const TypeDesc *chbegin, const TypeDesc *chend)
+{
+    clear ();
+    npixels = npix;
+    nchannels = nchan;
+    channeltypes.assign (chbegin, chend);
+    nsamples.resize (npixels, 0);
+    pointers.resize (size_t(npixels)*size_t(nchannels), NULL);
+}
+
+
+
+void
+DeepData::alloc ()
+{
+    // Calculate the total size we need, align each channel to 4 byte boundary
+    size_t totalsamples = 0, totalbytes = 0;
+    for (int i = 0;  i < npixels;  ++i) {
+        int s = nsamples[i];
+        totalsamples += s;
+        for (int c = 0;  c < nchannels;  ++c)
+            totalbytes += round_to_multiple (channeltypes[c].size() * s, 4);
+    }
+
+    // Set all the data pointers to the right offsets within the
+    // data block.  Leave the pointes NULL for pixels with no samples.
+    data.resize (totalbytes);
+    char *p = &data[0];
+    for (int i = 0;  i < npixels;  ++i) {
+        if (nsamples[i]) {
+            for (int c = 0;  c < nchannels;  ++c) {
+                pointers[i*nchannels+c] = p;
+                p += round_to_multiple (channeltypes[c].size()*nsamples[i], 4);
+            }
+        }
+    }
+}
+
+
+
+void
+DeepData::clear ()
+{
+    npixels = 0;
+    nchannels = 0;
+    channeltypes.clear();
+    nsamples.clear();
+    pointers.clear();
+    data.clear();
+}
+
+
+
+void
+DeepData::free ()
+{
+    std::vector<unsigned int>().swap (nsamples);
+    std::vector<void *>().swap (pointers);
+    std::vector<char>().swap (data);
+}
+
 }
 OIIO_NAMESPACE_EXIT
