@@ -45,7 +45,6 @@ using namespace std::tr1;
 #include "varyingref.h"
 #include "ustring.h"
 #include "filesystem.h"
-#include "hash.h"
 #include "thread.h"
 #include "fmath.h"
 #include "strutil.h"
@@ -119,8 +118,6 @@ iorate_compare (const ImageCacheFileRef &a, const ImageCacheFileRef &b)
 
 
 
-#ifdef OIIO_HAVE_BOOST_UNORDERED_MAP
-
 /// Perform "map[key] = value", and set sweep_iter = end() if it is invalidated.
 ///
 /// For some reason, unordered_map::insert and operator[] may invalidate
@@ -142,23 +139,6 @@ void safe_insert (HashMapT& map, const typename HashMapT::key_type& key,
         sweep_iter = map.end ();
 }
 
-#else
-
-template<typename HashMapT>
-void safe_insert (HashMapT& map, const typename HashMapT::key_type& key,
-                  const typename HashMapT::mapped_type& value,
-                  typename HashMapT::iterator& /*sweep_iter*/)
-{
-    // Traditional implementations of hash_map don't typically invalidate
-    // iterators on insertion.
-    //   - VC++'s stdext::hash_map, according to msdn, and 
-    //   - The implementation coming with g++ according to some vague
-    //     indications in the SGI docs & other places on the web.
-    map[key] = value;
-}
-
-#endif
-
 
 };  // end anonymous namespace
 
@@ -313,17 +293,21 @@ ImageCacheFile::open (ImageCachePerThreadInfo *thread_info)
     m_input.reset (ImageInput::create (m_filename.c_str(),
                                        m_imagecache.plugin_searchpath().c_str()));
     if (! m_input) {
-        imagecache().error ("%s", OIIO_NAMESPACE::geterror().c_str());
+        imagecache().error ("%s", OIIO::geterror().c_str());
         m_broken = true;
         invalidate_spec ();
         return false;
     }
 
+    ImageSpec configspec;
+    if (imagecache().unassociatedalpha())
+        configspec.attribute ("oiio:UnassociatedAlpha", 1);
+
     ImageSpec nativespec, tempspec;
     m_broken = false;
     bool ok = true;
     for (int tries = 0; tries <= imagecache().failure_retries(); ++tries) {
-        ok = m_input->open (m_filename.c_str(), nativespec);
+        ok = m_input->open (m_filename.c_str(), nativespec, configspec);
         if (ok) {
             tempspec = nativespec;
             if (tries)   // succeeded, but only after a failure!
@@ -355,6 +339,10 @@ ImageCacheFile::open (ImageCachePerThreadInfo *thread_info)
     // of the ImageCacheFile.
     m_subimages.clear ();
     int nsubimages = 0;
+
+    // Since each subimage can potentially have its own mipmap levels,
+    // keep track of the highest level discovered
+    int maxmip = 0;
     do {
         m_subimages.resize (nsubimages+1);
         SubimageInfo &si (subimageinfo(nsubimages));
@@ -379,6 +367,7 @@ ImageCacheFile::open (ImageCachePerThreadInfo *thread_info)
                     si.sscale = si.tscale = 1.0f;
                     si.soffset = si.toffset = 0.0f;
                 }
+                si.subimagename = ustring (tempspec.get_string_attribute("oiio:subimagename"));
             }
             if (tempspec.tile_width == 0 || tempspec.tile_height == 0) {
                 si.untiled = true;
@@ -414,9 +403,12 @@ ImageCacheFile::open (ImageCachePerThreadInfo *thread_info)
                 invalidate_spec ();
                 return false;
             }
+            // ImageCache can't store differing formats per channel
+            tempspec.channelformats.clear();
             LevelInfo levelinfo (tempspec, nativespec);
             si.levels.push_back (levelinfo);
             ++nmip;
+            maxmip = std::max (nmip, maxmip);
         } while (m_input->seek_subimage (nsubimages, nmip, nativespec));
 
         // Special work for non-MIPmapped images -- but only if "automip"
@@ -461,6 +453,7 @@ ImageCacheFile::open (ImageCachePerThreadInfo *thread_info)
                 s.tile_height = pow2roundup (s.tile_height);
                 s.tile_depth = pow2roundup (s.tile_depth);
                 ++nmip;
+                maxmip = std::max (nmip, maxmip);
                 LevelInfo levelinfo (s, s);
                 si.levels.push_back (levelinfo);
             }
@@ -563,11 +556,18 @@ ImageCacheFile::open (ImageCachePerThreadInfo *thread_info)
     // FIXME -- compute Mtex, Mras
 
     // See if there's a SHA-1 hash in the image description
-    std::string desc = spec.get_string_attribute ("ImageDescription");
-    const char *prefix = "SHA-1=";
-    size_t found = desc.rfind (prefix);
-    if (found != std::string::npos)
-        m_fingerprint = ustring (desc, found+strlen(prefix), 40);
+    std::string fing = spec.get_string_attribute ("oiio:SHA-1");
+    if (fing.length()) {
+        m_fingerprint = ustring(fing);
+    } else {
+        // If there was no "oiio:SHA-1" attribute, search for it as
+        // part of the ImageDescription.
+        std::string desc = spec.get_string_attribute ("ImageDescription");
+        const char *prefix = "SHA-1=";
+        size_t found = desc.rfind (prefix);
+        if (found != std::string::npos)
+            m_fingerprint = ustring (desc, found+strlen(prefix), 40);
+    }
 
     m_datatype = TypeDesc::FLOAT;
     if (! m_imagecache.forcefloat()) {
@@ -580,7 +580,10 @@ ImageCacheFile::open (ImageCachePerThreadInfo *thread_info)
     m_channelsize = m_datatype.size();
     m_pixelsize = m_channelsize * spec.nchannels;
     m_eightbit = (m_datatype == TypeDesc::UINT8);
-    m_mod_time = boost::filesystem::last_write_time (m_filename.string());
+    m_mod_time = Filesystem::last_write_time (m_filename.string());
+
+    // Set all mipmap level read counts to zero
+    m_mipreadcount.resize(maxmip, 0);
 
     DASSERT (! m_broken);
     m_validspec = true;
@@ -619,6 +622,9 @@ ImageCacheFile::read_tile (ImageCachePerThreadInfo *thread_info,
     if (miplevel > 0)
         m_mipused = true;
 
+    // count how many times this mipmap level was read
+    m_mipreadcount[miplevel]++;
+
     SubimageInfo &subinfo (subimageinfo(subimage));
 
     // Special case for un-MIP-mapped
@@ -662,7 +668,7 @@ ImageCacheFile::read_tile (ImageCachePerThreadInfo *thread_info,
             // TODO: should we attempt to close and re-open the file?
         }
         if (! ok)
-            imagecache().error ("%s", m_input->error_message().c_str());
+            imagecache().error ("%s", m_input->geterror().c_str());
     }
     if (ok) {
         size_t b = spec(subimage,miplevel).tile_bytes();
@@ -747,7 +753,8 @@ ImageCacheFile::read_unmipped (ImageCachePerThreadInfo *thread_info,
             ok &= imagecache().get_pixels (this, thread_info,
                                            subimage, miplevel-1,
                                            xlow, xlow+2, ylow, ylow+2,
-                                           0, 1, TypeDesc::FLOAT, bilerppels);
+                                           0, 1, 0, spec.nchannels,
+                                           TypeDesc::FLOAT, bilerppels);
             bilerp (bilerppels+0, bilerppels+spec.nchannels,
                     bilerppels+2*spec.nchannels, bilerppels+3*spec.nchannels,
                     xfrac, yfrac, spec.nchannels, resultpel);
@@ -756,7 +763,7 @@ ImageCacheFile::read_unmipped (ImageCachePerThreadInfo *thread_info,
     }
 
     // Now convert and copy those values out to the caller's buffer
-    lores.copy_pixels (0, tw, 0, th, format, data);
+    lores.get_pixels (0, tw, 0, th, 0, 1, format, data);
 
     // Restore the microcache to the way it was before.
     thread_info->tile = oldtile;
@@ -780,8 +787,10 @@ ImageCacheFile::read_untiled (ImageCachePerThreadInfo *thread_info,
     if (m_input->current_subimage() != subimage ||
         m_input->current_miplevel() != miplevel) {
         ImageSpec tmp;
-        if (! m_input->seek_subimage (subimage, miplevel, tmp))
+        if (! m_input->seek_subimage (subimage, miplevel, tmp)) {
+            imagecache().error ("%s", m_input->geterror().c_str());
             return false;
+        }
     }
 
     // We should not hold the tile mutex at this point
@@ -821,7 +830,7 @@ ImageCacheFile::read_untiled (ImageCachePerThreadInfo *thread_info,
         for (int scanline = y0, i = 0; scanline <= y1 && ok; ++scanline, ++i) {
             ok = m_input->read_scanline (scanline, z, format, (void *)&buf[scanlinesize*i]);
             if (! ok)
-                imagecache().error ("%s", m_input->error_message().c_str());
+                imagecache().error ("%s", m_input->geterror().c_str());
         }
         size_t b = (y1-y0+1) * spec.scanline_bytes();
         thread_info->m_stats.bytes_read += b;
@@ -870,7 +879,7 @@ ImageCacheFile::read_untiled (ImageCachePerThreadInfo *thread_info,
         // No auto-tile -- the tile is the whole image
         ok = m_input->read_image (format, data, xstride, ystride, zstride);
         if (! ok)
-            imagecache().error ("%s", m_input->error_message().c_str());
+            imagecache().error ("%s", m_input->geterror().c_str());
         size_t b = spec.image_bytes();
         thread_info->m_stats.bytes_read += b;
         m_bytesread += b;
@@ -936,6 +945,11 @@ ImageCacheImpl::find_file (ustring filename,
     ImageCacheFile *tf = NULL;
     bool newfile = false;
 
+    // Debugging aid: attribute "substitute_image" forces all image
+    // references to be to one named file.
+    if (m_substitute_image)
+        filename = m_substitute_image;
+
     // Part 1 - make sure the ImageCacheFile entry exists and is in the
     // file cache.  For this part, we need to lock the file cache.
     {
@@ -943,7 +957,7 @@ ImageCacheImpl::find_file (ustring filename,
         Timer timer;
 #endif
         DASSERT (m_filemutex_holder != thread_info);
-        ic_read_lock readguard (m_filemutex);
+        ic_write_lock readguard (m_filemutex);
         DASSERT (m_filemutex_holder == NULL);
         filemutex_holder (thread_info);
 #if IMAGECACHE_TIME_STATS
@@ -990,8 +1004,7 @@ ImageCacheImpl::find_file (ustring filename,
             // What if we've opened another file, with a different name,
             // but the SAME pixels?  It can happen!  Bad user, bad!  But
             // let's save them from their own foolishness.
-            bool was_duplicate = false;
-            if (tf->fingerprint ()) {
+            if (tf->fingerprint() && m_deduplicate) {
                 // std::cerr << filename << " hash=" << tf->fingerprint() << "\n";
                 ImageCacheFile *dup = find_fingerprint (tf->fingerprint(), tf);
                 if (dup != tf) {
@@ -1009,7 +1022,6 @@ ImageCacheImpl::find_file (ustring filename,
                         tf->m_sample_border == dup->m_sample_border) {
                         tf->duplicate (dup);
                         tf->close ();
-                        was_duplicate = true;
                         // std::cerr << "  duplicates " 
                         //   << fingerfound->second.get()->filename() << "\n";
                     }
@@ -1081,8 +1093,16 @@ ImageCacheImpl::check_max_files (ImageCachePerThreadInfo *thread_info)
         }
         if (m_file_sweep == m_files.end())   // If STILL at the end,
             break;                           //     it must be empty, done
-        ASSERT (full_loops < 100);  // abort rather than infinite loop
         DASSERT (m_file_sweep->second);
+        if (full_loops >= 100) {
+            // Somehow we've looped over the whole file list a lot of
+            // times, yet still haven't closed enough files to be below
+            // the limit on file handles.  Punt by breaking out of the
+            // loop, even though it may mean we exceed the limit on the
+            // number of open files that the user requested.
+            error ("Unable to free file handles fast enough");
+            break;
+        }
         m_file_sweep->second->release ();  // May reduce open files
         ++m_file_sweep;
     }
@@ -1097,7 +1117,7 @@ ImageCacheImpl::check_max_files_with_lock (ImageCachePerThreadInfo *thread_info)
     Timer timer;
 #endif
     DASSERT (m_filemutex_holder != thread_info);
-    ic_read_lock readguard (m_filemutex);
+    ic_write_lock readguard (m_filemutex);
     DASSERT (m_filemutex_holder == NULL);
     filemutex_holder (thread_info);
 #if IMAGECACHE_TIME_STATS
@@ -1204,12 +1224,9 @@ ImageCacheTile::read (ImageCachePerThreadInfo *thread_info)
 void
 ImageCacheTile::wait_pixels_ready () const
 {
+    atomic_backoff backoff;
     while (! m_pixels_ready) {
-        // Be kind to the CPU.  As far as I know, this does nothing on
-        // Windows.  Any Windows gurus know a better idea?
-#ifdef __TBB_Yield
-        __TBB_Yield ();
-#endif
+        backoff();
     }
 }
 
@@ -1257,6 +1274,8 @@ ImageCacheImpl::init ()
     m_accept_untiled = true;
     m_accept_unmipped = true;
     m_read_before_insert = false;
+    m_deduplicate = true;
+    m_unassociatedalpha = false;
     m_failure_retries = 0;
     m_latlong_y_up_default = true;
     m_Mw2c.makeIdentity();
@@ -1362,6 +1381,13 @@ ImageCacheImpl::onefile_stat_line (const ImageCacheFileRef &file,
                 break;
             }
     }
+    if (file->mipreadcount().size() > 1) {
+        out << " MIP-COUNT [";
+        int nmip = (int) file->mipreadcount().size();
+        for (int c = 0; c < nmip; c++)
+            out << (c ? "," : "") << file->mipreadcount()[c];
+        out << "]";
+    }
 
     return out.str ();
 }
@@ -1563,7 +1589,7 @@ ImageCacheImpl::reset_stats ()
     }
 
     {
-        ic_read_lock fileguard (m_filemutex);
+        ic_write_lock fileguard (m_filemutex);
         for (FilenameMap::const_iterator f = m_files.begin(); f != m_files.end(); ++f) {
             const ImageCacheFileRef &file (f->second);
             file->m_timesopened = 0;
@@ -1677,6 +1703,20 @@ ImageCacheImpl::attribute (const std::string &name, TypeDesc type,
             do_invalidate = true;
         }
     }
+    else if (name == "deduplicate" && type == TypeDesc::INT) {
+        int r = *(const int *)val;
+        if (r != m_deduplicate) {
+            m_deduplicate = r;
+            do_invalidate = true;
+        }
+    }
+    else if (name == "unassociatedalpha" && type == TypeDesc::INT) {
+        int r = *(const int *)val;
+        if (r != m_unassociatedalpha) {
+            m_unassociatedalpha = r;
+            do_invalidate = true;
+        }
+    }
     else if (name == "failure_retries" && type == TypeDesc::INT) {
         m_failure_retries = *(const int *)val;
     }
@@ -1686,6 +1726,9 @@ ImageCacheImpl::attribute (const std::string &name, TypeDesc type,
             m_latlong_y_up_default = y_up;
             do_invalidate = true;
         }
+    } else if (name == "substitute_image" && type == TypeDesc::STRING) {
+        m_substitute_image = ustring (*(const char **)val);
+        do_invalidate = true;
     } else {
         // Otherwise, unknown name
         return false;
@@ -1719,6 +1762,8 @@ ImageCacheImpl::getattribute (const std::string &name, TypeDesc type,
     ATTR_DECODE ("accept_untiled", int, m_accept_untiled);
     ATTR_DECODE ("accept_unmipped", int, m_accept_unmipped);
     ATTR_DECODE ("read_before_insert", int, m_read_before_insert);
+    ATTR_DECODE ("deduplicate", int, m_deduplicate);
+    ATTR_DECODE ("unassociatedalpha", int, m_unassociatedalpha);
     ATTR_DECODE ("failure_retries", int, m_failure_retries);
 
     // The cases that don't fit in the simple ATTR_DECODE scheme
@@ -1744,6 +1789,10 @@ ImageCacheImpl::getattribute (const std::string &name, TypeDesc type,
         *(const char **)val = ustring (m_latlong_y_up_default ? "y" : "z").c_str();
         return true;
     }
+    if (name == "substitute_image" && type == TypeDesc::STRING) {
+        *(const char **)val = m_substitute_image.c_str();
+        return true;
+    }
 
     // Stats we can just grab
     ATTR_DECODE ("stat:cache_memory_used", long long, m_mem_used);
@@ -1788,48 +1837,39 @@ ImageCacheImpl::find_tile_main_cache (const TileID &id, ImageCacheTileRef &tile,
 
     ++stats.find_tile_microcache_misses;
 
-    {
 #if IMAGECACHE_TIME_STATS
-        Timer timer;
+    Timer timer1;
 #endif
+    TileCache::iterator found;
+    {
         DASSERT (m_tilemutex_holder != thread_info); // shouldn't hold
         ic_read_lock readguard (m_tilemutex);
-        tilemutex_holder (thread_info);
+        // tilemutex_holder (thread_info);
 #if IMAGECACHE_TIME_STATS
-        stats.tile_locking_time += timer();
+        stats.tile_locking_time += timer1();
 #endif
-
-        TileCache::iterator found = m_tilecache.find (id);
+        found = m_tilecache.find (id);
 #if IMAGECACHE_TIME_STATS
-        stats.find_tile_time += timer();
+        stats.find_tile_time += timer1();
 #endif
-        if (found != m_tilecache.end()) {
+        if (found != m_tilecache.end())
             tile = found->second;
-            // We need to release the tile lock BEFORE calling
-            // wait_pixels_ready, or we could end up deadlocked if the
-            // other thread reading the pixels needs to lock the cache
-            // because it's doing automip.
-            DASSERT (m_tilemutex_holder == thread_info); // better still be us
-            tilemutex_holder (NULL);
-            m_tilemutex.unlock ();
-            tile->wait_pixels_ready ();
-            tile->use ();
-            DASSERT (id == tile->id());
-            DASSERT (tile);
-            DASSERT (m_tilemutex_holder != thread_info); // shouldn't hold
-            // Relock -- this shouldn't be necessary, but by golly, if I
-            // don't do this, I get inconsistent lock states.  Maybe a TBB
-            // bug in the lock_guard destructor if the spin mutex was 
-            // unlocked by hand, as above?
-            // FIXME -- try removing this if/when we upgrade to a newer TBB.
-            m_tilemutex.lock ();
-            return true;
-        }
-        DASSERT (tilemutex_holder() == thread_info); // better still be us
-        tilemutex_holder (NULL);
+        // DASSERT (m_tilemutex_holder == thread_info); // better still be us
+        // tilemutex_holder (NULL);
     }
 
-    DASSERT (m_tilemutex_holder != thread_info); // shouldn't hold
+    if (found != m_tilecache.end()) {
+        // We found the tile in the cache, but we need to make sure we
+        // wait until the pixels are ready to read.  We purposely have
+        // released the lock (above) before calling wait_pixels_ready,
+        // otherwise we could deadlock if another thread reading the
+        // pixels needs to lock the cache because it's doing automip.
+        tile->wait_pixels_ready ();
+        tile->use ();
+        DASSERT (id == tile->id());
+        DASSERT (tile);
+        return true;
+    }
 
     // The tile was not found in cache.
 
@@ -1933,7 +1973,15 @@ ImageCacheImpl::check_max_mem (ImageCachePerThreadInfo *thread_info)
         }
         if (m_tile_sweep == m_tilecache.end())  // If STILL at the end,
             break;                              //      it must be empty, done
-        ASSERT (full_loops < 100);  // abort rather than infinite loop
+        if (full_loops >= 100) {
+            // Somehow we've looped over the whole tile list a lot of
+            // times, yet still haven't freed enough tiles to be below
+            // the cache size limit.  Punt by breaking out of the loop,
+            // even though it may mean we exceed the cache size the user
+            // requested.
+            error ("Unable to free tiles fast enough");
+            break;
+        }
         if (! m_tile_sweep->second->release ()) {
             TileCache::iterator todelete = m_tile_sweep;
             ++m_tile_sweep;
@@ -2104,11 +2152,38 @@ ImageCacheImpl::imagespec (ustring filename, int subimage, int miplevel,
 
 
 
+int
+ImageCacheImpl::subimage_from_name (ImageCacheFile *file, ustring subimagename)
+{
+    for (int s = 0, send = file->subimages();  s < send;  ++s) {
+        if (file->subimageinfo(s).subimagename == subimagename)
+            return s;
+    }
+    return -1;  // No matching subimage name
+}
+
+
+
 bool
 ImageCacheImpl::get_pixels (ustring filename, int subimage, int miplevel,
                             int xbegin, int xend, int ybegin, int yend,
                             int zbegin, int zend,
                             TypeDesc format, void *result)
+{
+    return get_pixels (filename, subimage, miplevel,
+                       xbegin, xend, ybegin, yend, zbegin, zend,
+                       0, -1, format, result);
+}
+
+
+
+bool
+ImageCacheImpl::get_pixels (ustring filename,
+                            int subimage, int miplevel,
+                            int xbegin, int xend, int ybegin, int yend,
+                            int zbegin, int zend, int chbegin, int chend,
+                            TypeDesc format, void *result, stride_t xstride,
+                            stride_t ystride, stride_t zstride)
 {
     ImageCachePerThreadInfo *thread_info = get_perthread_info ();
     ImageCacheFile *file = find_file (filename, thread_info);
@@ -2131,8 +2206,10 @@ ImageCacheImpl::get_pixels (ustring filename, int subimage, int miplevel,
         return false;
     }
 
-    return get_pixels (file, thread_info, subimage, miplevel, xbegin, xend, 
-                       ybegin, yend, zbegin, zend, format, result);
+    return get_pixels (file, thread_info, subimage, miplevel,
+                       xbegin, xend, ybegin, yend, zbegin, zend,
+                       chbegin, chend, format, result,
+                       xstride, ystride, zstride);
 }
 
 
@@ -2142,58 +2219,107 @@ ImageCacheImpl::get_pixels (ImageCacheFile *file,
                             ImageCachePerThreadInfo *thread_info,
                             int subimage, int miplevel,
                             int xbegin, int xend, int ybegin, int yend,
-                            int zbegin, int zend, 
-                            TypeDesc format, void *result)
+                            int zbegin, int zend, int chbegin, int chend,
+                            TypeDesc format, void *result,
+                            stride_t xstride, stride_t ystride, stride_t zstride)
 {
     const ImageSpec &spec (file->spec(subimage, miplevel));
     bool ok = true;
 
-    // FIXME -- this could be WAY more efficient than starting from
-    // scratch for each pixel within the rectangle.  Instead, we should
-    // grab a whole tile at a time and memcpy it rapidly.  But no point
-    // doing anything more complicated (not to mention bug-prone) until
-    // somebody reports this routine as being a bottleneck.
-    int nc = spec.nchannels;
-    size_t formatpixelsize = nc * format.size();
-    size_t scanlinesize = (xend-xbegin) * formatpixelsize;
-    size_t zplanesize = (yend-ybegin) * scanlinesize;
+    // Compute channels and stride if not given (assume all channels,
+    // contiguous data layout for strides).
+    if (chbegin < 0 || chend < 0) {
+        chbegin = 0;
+        chend = spec.nchannels;
+    }
+    int nchans = chend - chbegin;
+    ImageSpec::auto_stride (xstride, ystride, zstride, format, nchans,
+                            xend-xbegin, yend-ybegin);
+
+    // formatpixelsize, scanlinesize, and zplanesize assume contiguous
+    // layout.  This may or may not be the same as the strides passed by
+    // the caller.
+    TypeDesc cachetype = file->datatype();
+    stride_t cache_stride = cachetype.size() * spec.nchannels;
+    size_t formatsize = format.size();
+    stride_t formatpixelsize = nchans * formatsize;
+    bool xcontig = (formatpixelsize == xstride && nchans == spec.nchannels);
+    stride_t scanlinesize = (xend-xbegin) * formatpixelsize;
+    stride_t zplanesize = (yend-ybegin) * scanlinesize;
     DASSERT (spec.depth >= 1 && spec.tile_depth >= 1);
-    for (int z = zbegin;  z < zend;  ++z) {
+
+    char *zptr = (char *)result;
+    for (int z = zbegin;  z < zend;  ++z, zptr += zstride) {
         if (z < spec.z || z >= (spec.z+spec.depth)) {
             // nonexistant planes
-            memset (result, 0, zplanesize);
-            result = (void *) ((char *) result + zplanesize);
+            if (xstride == formatpixelsize && ystride == scanlinesize) {
+                // Can zero out the plane in one shot
+                memset (zptr, 0, zplanesize);
+            } else {
+                // Non-contiguous strides -- zero out individual pixels
+                char *yptr = zptr;
+                for (int y = ybegin;  y < yend;  ++y, yptr += ystride) {
+                    char *xptr = yptr;
+                    for (int x = xbegin;  x < xend;  ++x, xptr += xstride)
+                        memset (xptr, 0, formatpixelsize);
+                }
+            }
             continue;
         }
         int tz = z - ((z - spec.z) % spec.tile_depth);
-        for (int y = ybegin;  y < yend;  ++y) {
+        char *yptr = zptr;
+        for (int y = ybegin;  y < yend;  ++y, yptr += ystride) {
             if (y < spec.y || y >= (spec.y+spec.height)) {
                 // nonexistant scanlines
-                memset (result, 0, scanlinesize);
-                result = (void *) ((char *) result + scanlinesize);
+                if (xstride == formatpixelsize) {
+                    // Can zero out the scanline in one shot
+                    memset (yptr, 0, scanlinesize);
+                } else {
+                    // Non-contiguous strides -- zero out individual pixels
+                    char *xptr = yptr;
+                    for (int x = xbegin;  x < xend;  ++x, xptr += xstride)
+                        memset (xptr, 0, formatpixelsize);
+                }
                 continue;
             }
             int ty = y - ((y - spec.y) % spec.tile_height);
-            for (int x = xbegin;  x < xend;  ++x) {
+            char *xptr = yptr;
+            int old_tx = -100000;
+            const char *data = NULL;
+            for (int x = xbegin;  x < xend;  ++x, xptr += xstride) {
                 if (x < spec.x || x >= (spec.x+spec.width)) {
                     // nonexistant columns
-                    memset (result, 0, formatpixelsize);
-                    result = (void *) ((char *) result + formatpixelsize);
+                    memset (xptr, 0, formatpixelsize);
                     continue;
                 }
                 int tx = x - ((x - spec.x) % spec.tile_width);
-                TileID tileid (*file, subimage, miplevel, tx, ty, tz);
-                ok &= find_tile (tileid, thread_info);
-                if (! ok)
-                    return false;  // Just stop if file read failed
-                ImageCacheTileRef &tile (thread_info->tile);
-                const char *data;
-                if (tile && (data = (const char *)tile->data (x, y, z))) {
-                    convert_types (file->datatype(), data, format, result, nc);
+                if (old_tx != tx) {
+                    // Only do a find_tile and re-setup of the data
+                    // pointer when we move across a tile boundary.
+                    TileID tileid (*file, subimage, miplevel, tx, ty, tz);
+                    ok &= find_tile (tileid, thread_info);
+                    if (! ok)
+                        return false;  // Just stop if file read failed
+                    ImageCacheTileRef &tile (thread_info->tile);
+                    ASSERT (tile);
+                    data = (const char *)tile->data (x, y, z)
+                                        + chbegin*formatsize;
+                    ASSERT (data);
+                    old_tx = tx;
+                }
+                if (xcontig) {
+                    // Special case for a contiguous span within one tile
+                    int spanend = std::min (tx + spec.tile_width, xend);
+                    stride_t span = spanend - x;
+                    convert_types (cachetype, data, format, xptr, nchans*span);
+                    x += (span-1);
+                    xptr += xstride * (span-1);
+                    // no need to increment data, since next read will
+                    // be from a different tile
                 } else {
-                    memset (result, 0, formatpixelsize);
+                    convert_types (cachetype, data, format, xptr, nchans);
+                    data += cache_stride;
                 }
-                result = (void *) ((char *) result + formatpixelsize);
             }
         }
     }
@@ -2262,7 +2388,7 @@ ImageCacheImpl::invalidate (ustring filename)
 {
     ImageCacheFile *file = NULL;
     {
-        ic_read_lock fileguard (m_filemutex);
+        ic_write_lock fileguard (m_filemutex);
         FilenameMap::iterator fileit = m_files.find (filename);
         if (fileit != m_files.end()) {
             file = fileit->second.get();
@@ -2321,7 +2447,7 @@ ImageCacheImpl::invalidate_all (bool force)
     // Make a list of all files that need to be invalidated
     std::vector<ustring> all_files;
     {
-        ic_read_lock fileguard (m_filemutex);
+        ic_write_lock fileguard (m_filemutex);
         for (FilenameMap::iterator fileit = m_files.begin();
                  fileit != m_files.end();  ++fileit) {
             ImageCacheFileRef &f (fileit->second);
@@ -2331,7 +2457,7 @@ ImageCacheImpl::invalidate_all (bool force)
                 all_files.push_back (name);
                 continue;
             }
-            std::time_t t = boost::filesystem::last_write_time (name.string());
+            std::time_t t = Filesystem::last_write_time (name.string());
             // Invalidate the file if it has been modified since it was
             // last opened, or if 'force' is true.
             bool inval = force || (t != f->mod_time());
@@ -2450,7 +2576,7 @@ ImageCacheImpl::geterror () const
 
 
 void
-ImageCacheImpl::error (const char *message, ...)
+ImageCacheImpl::append_error (const std::string& message) const
 {
     std::string *errptr = m_errormessage.get ();
     if (! errptr) {
@@ -2462,10 +2588,7 @@ ImageCacheImpl::error (const char *message, ...)
             "Accumulated error messages > 16MB. Try checking return codes!");
     if (errptr->size())
         *errptr += '\n';
-    va_list ap;
-    va_start (ap, message);
-    *errptr += Strutil::vformat (message, ap);
-    va_end (ap);
+    *errptr += message;
 }
 
 
