@@ -36,8 +36,11 @@
 #ifndef OPENIMAGEIO_IMAGECACHE_PVT_H
 #define OPENIMAGEIO_IMAGECACHE_PVT_H
 
+#include <boost/unordered_map.hpp>
+
 #include "texture.h"
 #include "refcnt.h"
+#include "hash.h"
 
 
 OIIO_NAMESPACE_ENTER
@@ -53,6 +56,8 @@ namespace pvt {
 # define IMAGECACHE_TIME_STATS 0
 #endif
 
+#define IMAGECACHE_USE_RW_MUTEX 1
+
 
 class ImageCacheImpl;
 struct ImageCachePerThreadInfo;
@@ -173,6 +178,7 @@ public:
     size_t pixelsize () const { return m_pixelsize; }
     bool eightbit (void) const { return m_eightbit; }
     bool mipused (void) const { return m_mipused; }
+    const std::vector<size_t> &mipreadcount (void) const { return m_mipreadcount; }
 
     void invalidate ();
 
@@ -210,6 +216,7 @@ public:
         // 0-1 texture space relative to the "display/full window" into 
         // 0-1 relative to the "pixel window".
         float sscale, soffset, tscale, toffset;
+        ustring subimagename;
 
         SubimageInfo () : untiled(false), unmipped(false) { }
         ImageSpec &spec (int m) { return levels[m].spec; }
@@ -274,6 +281,7 @@ private:
     size_t m_timesopened;           ///< Separate times we opened this file
     double m_iotime;                ///< I/O time for this file
     bool m_mipused;                 ///< MIP level >0 accessed
+    std::vector<size_t> m_mipreadcount; ///< Tile reads per mip level
     volatile bool m_validspec;      ///< If false, reread spec upon open
     ImageCacheImpl &m_imagecache;   ///< Back pointer for ImageCache
     mutable recursive_mutex m_input_mutex; ///< Mutex protecting the ImageInput
@@ -312,19 +320,11 @@ private:
                         TypeDesc format, void *data);
 
     void lock_input_mutex () {
-#if (BOOST_VERSION >= 103500)
         m_input_mutex.lock ();
-#else
-        boost::detail::thread::lock_ops<recursive_mutex>::lock (m_input_mutex);
-#endif
     }
 
     void unlock_input_mutex () {
-#if (BOOST_VERSION >= 103500)
         m_input_mutex.unlock ();
-#else
-        boost::detail::thread::lock_ops<recursive_mutex>::unlock (m_input_mutex);
-#endif
     }
 
     friend class ImageCacheImpl;
@@ -340,11 +340,7 @@ typedef intrusive_ptr<ImageCacheFile> ImageCacheFileRef;
 
 /// Map file names to file references
 ///
-#ifdef OIIO_HAVE_BOOST_UNORDERED_MAP
 typedef boost::unordered_map<ustring,ImageCacheFileRef,ustringHash> FilenameMap;
-#else
-typedef hash_map<ustring,ImageCacheFileRef,ustringHash> FilenameMap;
-#endif
 
 
 
@@ -376,10 +372,6 @@ public:
     int y (void) const { return m_y; }
     int z (void) const { return m_z; }
 
-    void x (int v) { m_x = v; }
-    void y (int v) { m_y = v; }
-    void z (int v) { m_z = v; }
-
     /// Do the two ID's refer to the same tile?  
     ///
     friend bool equal (const TileID &a, const TileID &b) {
@@ -403,13 +395,19 @@ public:
     ///
     bool operator== (const TileID &b) const { return equal (*this, b); }
 
-    /// Digest the TileID into a size_t to use as a hash key.  We do
-    /// this by multiplying each element by a different prime and
-    /// summing, so that collisions are unlikely.
+    /// Digest the TileID into a size_t to use as a hash key.
     size_t hash () const {
+#if 0
+        // original -- turned out not to fill hash buckets evenly
         return m_x * 53 + m_y * 97 + m_z * 193 + 
                m_subimage * 389 + m_miplevel * 1543 +
                m_file.filename().hash() * 769;
+#else
+        // Good compromise!
+        return bjhash::bjfinal (m_x+1543, m_y + 6151 + m_z*769,
+                                m_miplevel + (m_subimage<<8))
+                           + m_file.filename().hash();
+#endif
     }
 
     /// Functor that hashes a TileID
@@ -530,11 +528,8 @@ typedef intrusive_ptr<ImageCacheTile> ImageCacheTileRef;
 
 /// Hash table that maps TileID to ImageCacheTileRef -- this is the type of the
 /// main tile cache.
-#ifdef OIIO_HAVE_BOOST_UNORDERED_MAP
 typedef boost::unordered_map<TileID, ImageCacheTileRef, TileID::Hasher> TileCache;
-#else
-typedef hash_map<TileID, ImageCacheTileRef, TileID::Hasher> TileCache;
-#endif
+
 
 /// A very small amount of per-thread data that saves us from locking
 /// the mutex quite as often.  We store things here used by both
@@ -649,6 +644,7 @@ public:
     bool forcefloat () const { return m_forcefloat; }
     bool accept_untiled () const { return m_accept_untiled; }
     bool accept_unmipped () const { return m_accept_unmipped; }
+    bool unassociatedalpha () const { return m_unassociatedalpha; }
     int failure_retries () const { return m_failure_retries; }
     bool latlong_y_up_default () const { return m_latlong_y_up_default; }
     void get_commontoworld (Imath::M44f &result) const {
@@ -679,13 +675,22 @@ public:
                              int xbegin, int xend,
                              int ybegin, int yend, int zbegin, int zend,
                              TypeDesc format, void *result);
+    virtual bool get_pixels (ustring filename,
+                    int subimage, int miplevel, int xbegin, int xend,
+                    int ybegin, int yend, int zbegin, int zend,
+                    int chbegin, int chend, TypeDesc format, void *result,
+                    stride_t xstride=AutoStride, stride_t ystride=AutoStride,
+                    stride_t zstride=AutoStride);
 
     /// Retrieve a rectangle of raw unfiltered pixels, from an open valid
     /// ImageCacheFile.
-    bool get_pixels (ImageCacheFile *file, ImageCachePerThreadInfo *thread_info,
-                     int subimage, int miplevel, int xmin, int xmax,
-                     int ymin, int ymax, int zmin, int zmax, 
-                     TypeDesc format, void *result);
+    bool get_pixels (ImageCacheFile *file,
+                     ImageCachePerThreadInfo *thread_info,
+                     int subimage, int miplevel, int xbegin, int xend,
+                     int ybegin, int yend, int zbegin, int zend,
+                     int chbegin, int chend, TypeDesc format, void *result,
+                     stride_t xstride=AutoStride, stride_t ystride=AutoStride,
+                     stride_t zstride=AutoStride);
 
     /// Find the ImageCacheFile record for the named image, or NULL if
     /// no such file can be found.  This returns a plain old pointer,
@@ -706,11 +711,11 @@ public:
             ic_read_lock lock (m_tilemutex);
 #ifdef DEBUG
             DASSERT (m_tilemutex_holder == NULL);
-            m_tilemutex_holder = thread_info;
+//            m_tilemutex_holder = thread_info;
 #endif
             found = m_tilecache.find (id);
 #ifdef DEBUG
-            m_tilemutex_holder = NULL;
+//            m_tilemutex_holder = NULL;
 #endif
         } else {
             // Caller already holds the lock
@@ -760,6 +765,11 @@ public:
     virtual void release_tile (Tile *tile) const;
     virtual const void * tile_pixels (Tile *tile, TypeDesc &format) const;
 
+    /// Return the numerical subimage index for the given subimage name,
+    /// as stored in the "oiio:subimagename" metadata.  Return -1 if no
+    /// subimage matches its name.
+    int subimage_from_name (ImageCacheFile *file, ustring subimagename);
+
     virtual std::string geterror () const;
     virtual std::string getstats (int level=1) const;
     virtual void reset_stats ();
@@ -814,7 +824,12 @@ public:
 
     /// Internal error reporting routine, with printf-like arguments.
     ///
-    void error (const char *message, ...) OPENIMAGEIO_PRINTF_ARGS(2,3);
+    /// void error (const char *message, ...);
+    TINYFORMAT_WRAP_FORMAT (void, error, const,
+        std::ostringstream msg;, msg, append_error(msg.str());)
+
+    /// Append a string to the current error message
+    void append_error (const std::string& message) const;
 
     /// Get a pointer to the caller's thread's per-thread info, or create
     /// one in the first place if there isn't one already.
@@ -901,9 +916,15 @@ private:
     /// Clear the fingerprint list, thread-safe.
     void clear_fingerprints ();
 
+#if IMAGECACHE_USE_RW_MUTEX
+    typedef spin_rw_mutex ic_mutex;
+    typedef spin_rw_write_lock ic_read_lock;
+    typedef spin_rw_write_lock ic_write_lock;
+#else
     typedef spin_mutex ic_mutex;
     typedef spin_lock  ic_read_lock;
     typedef spin_lock  ic_write_lock;
+#endif
 
     thread_specific_ptr< ImageCachePerThreadInfo > m_perthread_info;
     std::vector<ImageCachePerThreadInfo *> m_all_perthread_info;
@@ -920,10 +941,13 @@ private:
     bool m_accept_untiled;       ///< Accept untiled images?
     bool m_accept_unmipped;      ///< Accept unmipped images?
     bool m_read_before_insert;   ///< Read tiles before adding to cache?
+    bool m_deduplicate;          ///< Detect duplicate files?
+    bool m_unassociatedalpha;    ///< Keep unassociated alpha files as they are?
     int m_failure_retries;       ///< Times to re-try disk failures
     bool m_latlong_y_up_default; ///< Is +y the default "up" for latlong?
     Imath::M44f m_Mw2c;          ///< world-to-"common" matrix
     Imath::M44f m_Mc2w;          ///< common-to-world matrix
+    ustring m_substitute_image;  ///< Substitute this image for all others
 
     mutable ic_mutex m_filemutex; ///< Thread safety for file cache
     FilenameMap m_files;         ///< Map file names to ImageCacheFile's
