@@ -62,6 +62,8 @@ class ImageCacheTileRef;
 ///
 class TextureSystemImpl : public TextureSystem {
 public:
+    typedef ImageCacheFile TextureFile;
+
     TextureSystemImpl (ImageCache *imagecache);
     virtual ~TextureSystemImpl ();
 
@@ -264,7 +266,6 @@ public:
     void operator delete (void *todel) { ::delete ((char *)todel); }
 
 private:
-    typedef ImageCacheFile TextureFile;
     typedef ImageCacheTileRef TileRef;
     typedef ImageCachePerThreadInfo PerThreadInfo;
 
@@ -395,8 +396,8 @@ private:
     /// ratio is returned (possibly adjusting major and minorlength to
     /// conform to the aniso limits) but the true aspect is stored in
     /// 'trueaspect'.
-    float anisotropic_aspect (float &majorlength, float &minorlength,
-                              TextureOpt& options, float &trueaspect);
+    static float anisotropic_aspect (float &majorlength, float &minorlength,
+                                     TextureOpt& options, float &trueaspect);
 
     /// Convert texture coordinates (s,t), which range on 0-1 for the
     /// "full" image boundary, to texel coordinates (i+ifrac,j+jfrac)
@@ -414,7 +415,7 @@ private:
 
     /// Correctly fill in channels that were requested but not present
     /// in the file.
-    void fill_channels (int nfilechannels, TextureOpt &options, float *result);
+    void fill_channels (const ImageSpec &spec, TextureOpt &options, float *result);
 
     typedef bool (*wrap_impl) (int &coord, int origin, int width);
     static bool wrap_black (int &coord, int origin, int width);
@@ -443,12 +444,24 @@ private:
                        const ImageCacheFile::LevelInfo &levelinfo,
                        TextureOpt &options, int miplevel, int nchannels);
 
+    /// Perform short unit tests.
+    void unit_test_texture ();
+
     /// Internal error reporting routine, with printf-like arguments.
     ///
-    void error (const char *message, ...) OPENIMAGEIO_PRINTF_ARGS(2,3);
+    /// void error (const char *message, ...) const
+    TINYFORMAT_WRAP_FORMAT (void, error, const,
+        std::ostringstream msg;, msg, append_error(msg.str());)
+
+    /// Append a string to the current error message
+    void append_error (const std::string& message) const;
 
     void printstats () const;
 
+    // Debugging aid
+    void visualize_ellipse (const std::string &name, float dsdx, float dtdx,
+                            float dsdy, float dtdy, float sblur, float tblur);
+
     ImageCacheImpl *m_imagecache;
     Imath::M44f m_Mw2c;          ///< world-to-"common" matrix
     Imath::M44f m_Mc2w;          ///< common-to-world matrix
@@ -483,10 +496,20 @@ TextureSystemImpl::anisotropic_aspect (float &majorlength, float &minorlength,
         //    it slightly too blurry along the minor axis, slightly
         //    aliasing along the major axis.  You can't please everybody.
         if (options.conservative_filter) {
+#if 0
+            // Soltion (a) -- never alias by blurring more along minor axis
+            minorlength = majorlength / options.anisotropic;
+#else
             // Solution (c) -- this is our default, usually a nice balance.
-            majorlength = sqrtf ((majorlength) *
+            // We used to take the geometric mean...
+//            majorlength = sqrtf ((majorlength) *
+//                                 (minorlength * options.anisotropic));
+            // ...but frankly I find the average to be a little more
+            // visually pleasing.
+            majorlength = 0.5f * ((majorlength) +
                                  (minorlength * options.anisotropic));
             minorlength = majorlength / options.anisotropic;
+#endif
         } else {
             // Solution (b) -- alias slightly, never overblur
             majorlength = minorlength * options.anisotropic;
