@@ -38,6 +38,7 @@
 #include <string>
 #include <sstream>
 #include <utility>
+#include <ctype.h>
 
 #include <boost/tokenizer.hpp>
 #include <boost/foreach.hpp>
@@ -140,11 +141,23 @@ Oiiotool::process_pending ()
 
 
 
+void
+Oiiotool::error (const std::string &command, const std::string &explanation)
+{
+    std::cerr << "ERROR: " << command;
+    if (explanation.length())
+        std::cerr << " (" << explanation << ")";
+    std::cerr << "\n";
+    exit (-1);
+}
+
+
+
 static int
 set_threads (int argc, const char *argv[])
 {
-    ASSERT (argc == 1);
-    OIIO_NAMESPACE::attribute ("threads", atoi(argv[0]));
+    ASSERT (argc == 2);
+    OIIO::attribute ("threads", atoi(argv[1]));
     return 0;
 }
 
@@ -210,6 +223,19 @@ adjust_output_options (ImageSpec &spec, const Oiiotool &ot)
     if (ot.output_planarconfig == "contig" ||
         ot.output_planarconfig == "separate")
         spec.attribute ("planarconfig", ot.output_planarconfig);
+
+    // Append command to image history
+    std::string history = spec.get_string_attribute ("Exif:ImageHistory");
+    if (! Strutil::iends_with (history, ot.full_command_line)) { // don't add twice
+        if (history.length() && ! Strutil::iends_with (history, "\n"))
+            history += std::string("\n");
+        history += ot.full_command_line;
+        spec.attribute ("Exif:ImageHistory", history);
+    }
+
+    std::string software = Strutil::format ("OpenImageIO %s : %s",
+                                   OIIO_VERSION_STRING, ot.full_command_line);
+    spec.attribute ("Software", software);
 }
 
 
@@ -275,14 +301,37 @@ output_file (int argc, const char *argv[])
         ir = ot.curimg;
     }
 
-    ImageOutput::OpenMode mode = ImageOutput::Create;  // initial open
+    std::vector<ImageSpec> subimagespecs (ir->subimages());
+    for (int s = 0;  s < ir->subimages();  ++s) {
+        ImageSpec spec = *ir->spec(s,0);
+        adjust_output_options (spec, ot);
+        subimagespecs[s] = spec;
+    }
+
+    // Do the initial open
+    ImageOutput::OpenMode mode = ImageOutput::Create;
+    if (ir->subimages() > 1 && out->supports("multiimage")) {
+        if (! out->open (filename, ir->subimages(), &subimagespecs[0])) {
+            std::cerr << "oiiotool ERROR: " << out->geterror() << "\n";
+            return 0;
+        }
+    } else {
+        if (! out->open (filename, subimagespecs[0], mode)) {
+            std::cerr << "oiiotool ERROR: " << out->geterror() << "\n";
+            return 0;
+        }
+    }
+
+    // Output all the subimages and MIP levels
     for (int s = 0, send = ir->subimages();  s < send;  ++s) {
         for (int m = 0, mend = ir->miplevels(s);  m < mend;  ++m) {
             ImageSpec spec = *ir->spec(s,m);
             adjust_output_options (spec, ot);
-            if (! out->open (filename, spec, mode)) {
-                std::cerr << "oiiotool ERROR: " << out->geterror() << "\n";
-                return 0;
+            if (s > 0 || m > 0) {  // already opened first subimage/level
+                if (! out->open (filename, spec, mode)) {
+                    std::cerr << "oiiotool ERROR: " << out->geterror() << "\n";
+                    return 0;
+                }
             }
             if (! (*ir)(s,m).write (out)) {
                 std::cerr << "oiiotool ERROR: " << (*ir)(s,m).geterror() << "\n";
@@ -318,7 +367,7 @@ output_file (int argc, const char *argv[])
         std::time_t in_time = ir->time();
         if (! metadatatime.empty())
             DateTime_to_time_t (metadatatime.c_str(), in_time);
-        boost::filesystem::last_write_time (filename, in_time);
+        Filesystem::last_write_time (filename, in_time);
     }
 
     ot.curimg = saveimg;
@@ -764,7 +813,9 @@ action_colorconvert (int argc, const char *argv[])
 
     for (int s = 0, send = A->subimages();  s < send;  ++s) {
         for (int m = 0, mend = A->miplevels(s);  m < mend;  ++m) {
-            ImageBufAlgo::colorconvert ((*ot.curimg)(s,m), (*A)(s,m), processor, false);
+            bool ok = ImageBufAlgo::colorconvert ((*ot.curimg)(s,m), (*A)(s,m), processor, false);
+            if (! ok)
+                ot.error (argv[0], (*ot.curimg)(s,m).geterror());
             ot.curimg->spec(s,m)->attribute ("oiio::Colorspace", tospace);
         }
     }
@@ -822,6 +873,126 @@ action_unmip (int argc, const char *argv[])
 
 
 
+// For a given spec (which contains the channel names for an image), and
+// a comma separated list of channels (e.g., "B,G,R,A"), compute the
+// vector of integer indices for those channels (e.g., {2,1,0,3}).
+// Return true for success, false for failure, including if any of the
+// channels were not present in the image.
+static bool
+decode_channel_set (const ImageSpec &spec, std::string chanlist,
+                    std::vector<std::string> &channelnames,
+                    std::vector<int> &channels, std::vector<float> &values)
+{
+    channels.clear ();
+    while (chanlist.length()) {
+        // Extract the next channel name
+        size_t pos = chanlist.find_first_of(",");
+        std::string onechan (chanlist, 0, pos);
+        onechan = Strutil::strip (onechan);
+        if (pos == std::string::npos)
+            chanlist.clear();
+        else
+            chanlist = chanlist.substr (pos+1, std::string::npos);
+
+        // Find the index corresponding to that channel
+        float value = 0.0f;
+        int ch = -1;
+        for (int i = 0;  i < spec.nchannels;  ++i)
+            if (spec.channelnames[i] == onechan) {
+                ch = i;  break;
+            }
+        if (ch < 0 && onechan.length() &&
+                (isdigit(onechan[0]) || onechan[0] == '-'))
+            ch = atoi (onechan.c_str());
+        if (ch < 0 && onechan.length() && onechan[0] == '=')
+            value = atof (onechan.c_str()+1);
+        channelnames.push_back (onechan);
+        channels.push_back (ch);
+        values.push_back (value);
+    }
+    return true;
+}
+
+
+
+static int
+action_channels (int argc, const char *argv[])
+{
+    if (ot.postpone_callback (1, action_channels, argc, argv))
+        return 0;
+
+    ImageRecRef A (ot.pop());
+    ot.read (A);
+
+    std::string chanlist = argv[1];
+    if (chanlist == "RGB")   // Fix common synonyms/mistakes
+        chanlist = "R,G,B";
+    else if (chanlist == "RGBA")
+        chanlist = "R,G,B,A";
+
+    // Decode the channel set, make the full list of ImageSpec's we'll
+    // need to describe the new ImageRec with the altered channels.
+    std::vector<int> allmiplevels;
+    std::vector<ImageSpec> allspecs;
+    for (int s = 0, subimages = ot.allsubimages ? A->subimages() : 1;
+         s < subimages;  ++s) {
+        std::vector<std::string> channelnames;
+        std::vector<int> channels;
+        std::vector<float> values;
+        bool ok = decode_channel_set (*A->spec(s,0), chanlist,
+                                      channelnames, channels, values);
+        if (! ok) {
+            ot.error (argv[0], Strutil::format("Invalid or unknown channel selection \"%s\"", chanlist));
+            ot.push (A);
+            return 0;
+        }
+        int miplevels = ot.allsubimages ? A->miplevels(s) : 1;
+        allmiplevels.push_back (miplevels);
+        for (int m = 0;  m < miplevels;  ++m) {
+            ImageSpec spec = *A->spec(s,m);
+            spec.nchannels = (int)channelnames.size();
+            spec.default_channel_names ();
+            spec.channelformats.clear();
+            allspecs.push_back (spec);
+        }
+    }
+
+    // Create the replacement ImageRec
+    ImageRecRef R (new ImageRec(A->name(), (int)allmiplevels.size(),
+                                &allmiplevels[0], &allspecs[0]));
+    ot.push (R);
+
+    // Subimage by subimage, MIP level by MIP level, copy/shuffle the
+    // channels individually from the source image into the result.
+    for (int s = 0, subimages = R->subimages();  s < subimages;  ++s) {
+        std::vector<std::string> channelnames;
+        std::vector<int> channels;
+        std::vector<float> values;
+        decode_channel_set (*A->spec(s,0), chanlist, channelnames,
+                            channels, values);
+        for (int m = 0, miplevels = R->miplevels(s);  m < miplevels;  ++m) {
+            // Shuffle the indexed/named channels
+            ImageBufAlgo::channels ((*R)(s,m), (*A)(s,m), (int)channels.size(),
+                                    &channels[0], false);
+            // Set channels that are literals
+            for (int c = 0;  c < (int)channels.size();  ++c)
+                if (channels[c] < 0) {
+                    ROI roi = get_roi (*R->spec(s,m));
+                    roi.chbegin = c;
+                    roi.chend = c+1;
+                    ImageBufAlgo::fill ((*R)(s,m), &values[c], roi);
+                }
+            // Tricky subtlety: IBA::channels changed the underlying IB,
+            // we may need to update the IRR's copy of the spec.
+            R->update_spec_from_imagebuf(s,m);
+        }
+    }
+
+    return 0;
+}
+
+
+
 static int
 action_selectmip (int argc, const char *argv[])
 {
@@ -901,7 +1072,9 @@ action_add (int argc, const char *argv[])
                 continue;
             }
             ImageBuf &Rib ((*ot.curimg)(s,m));
-            ImageBufAlgo::add (Rib, Aib, Bib);
+            bool ok = ImageBufAlgo::add (Rib, Aib, Bib);
+            if (! ok)
+                ot.error (argv[0], Rib.geterror());
         }
     }
              
@@ -1133,7 +1306,9 @@ action_create (int argc, const char *argv[])
     spec.full_height = spec.height;
     spec.full_depth = spec.depth;
     ImageRecRef img (new ImageRec ("new", spec, ot.imagecache));
-    ImageBufAlgo::zero ((*img)());
+    bool ok = ImageBufAlgo::zero ((*img)());
+    if (! ok)
+        ot.error (argv[0], (*img)().geterror());
     if (ot.curimg)
         ot.image_stack.push_back (ot.curimg);
     ot.curimg = img;
@@ -1163,7 +1338,31 @@ action_pattern (int argc, const char *argv[])
     ImageBuf &ib ((*img)());
     std::string pattern = argv[1];
     if (Strutil::iequals(pattern,"black")) {
-        ImageBufAlgo::zero (ib);
+        bool ok = ImageBufAlgo::zero (ib);
+        if (! ok)
+            ot.error (argv[0], ib.geterror());
+    } else if (Strutil::istarts_with(pattern,"constant")) {
+        float *fill = ALLOCA (float, nchans);
+        for (int c = 0;  c < nchans;  ++c)
+            fill[c] = 1.0f;
+        size_t pos;
+        while ((pos = pattern.find_first_of(":")) != std::string::npos) {
+            pattern = pattern.substr (pos+1, std::string::npos);
+            if (Strutil::istarts_with(pattern,"color=")) {
+                // Parse comma-separated color list
+                size_t numpos = 6;
+                for (int c = 0; c < nchans && numpos < pattern.size() && pattern[numpos] != ':'; ++c) {
+                    fill[c] = atof (pattern.c_str()+numpos);
+                    while (numpos < pattern.size() && pattern[numpos] != ':' && pattern[numpos] != ',')
+                        ++numpos;
+                    if (numpos < pattern.size())
+                        ++numpos;
+                }
+            }
+        }
+        bool ok = ImageBufAlgo::fill (ib, fill);
+        if (! ok)
+            ot.error (argv[0], ib.geterror());
     } else if (Strutil::istarts_with(pattern,"checker")) {
         int width = 8;
         size_t pos;
@@ -1174,16 +1373,44 @@ action_pattern (int argc, const char *argv[])
         }
         std::vector<float> color1 (nchans, 0.0f);
         std::vector<float> color2 (nchans, 1.0f);
-        ImageBufAlgo::checker (ib, width, &color1[0], &color2[0],
-                               ib.xbegin(), ib.xend(),
-                               ib.ybegin(), ib.yend(),
-                               ib.zbegin(), ib.zend());
+        bool ok = ImageBufAlgo::checker (ib, width, &color1[0], &color2[0],
+                                         ib.xbegin(), ib.xend(),
+                                         ib.ybegin(), ib.yend(),
+                                         ib.zbegin(), ib.zend());
+        if (! ok)
+            ot.error (argv[0], ib.geterror());
     } else {
-        ImageBufAlgo::zero (ib);
+        bool ok = ImageBufAlgo::zero (ib);
+        if (! ok)
+            ot.error (argv[0], ib.geterror());
     }
-    if (ot.curimg)
-        ot.image_stack.push_back (ot.curimg);
-    ot.curimg = img;
+    ot.push (img);
+    return 0;
+}
+
+
+
+static int
+action_capture (int argc, const char *argv[])
+{
+    ASSERT (argc == 1);
+    int camera = 0;
+
+    std::string cmd = argv[0];
+    size_t pos;
+    while ((pos = cmd.find_first_of(":")) != std::string::npos) {
+        cmd = cmd.substr (pos+1, std::string::npos);
+        if (Strutil::istarts_with(cmd,"camera="))
+            camera = atoi(cmd.c_str()+7);
+    }
+
+    ImageBuf ib;
+    bool ok = ImageBufAlgo::capture_image (ib, camera, TypeDesc::FLOAT);
+    if (! ok)
+        ot.error (argv[0], ib.geterror());
+    ImageRecRef img (new ImageRec ("capture", ib.spec(), ot.imagecache));
+    (*img)().copy (ib);
+    ot.push (img);
     return 0;
 }
 
@@ -1208,8 +1435,10 @@ action_crop (int argc, const char *argv[])
         ot.push (new ImageRec (A->name(), newspec, ot.imagecache));
         const ImageBuf &Aib ((*A)(0,0));
         ImageBuf &Rib ((*ot.curimg)(0,0));
-        ImageBufAlgo::crop (Rib, Aib, newspec.x, newspec.x+newspec.width,
-                            newspec.y, newspec.y+newspec.height);
+        bool ok = ImageBufAlgo::crop (Rib, Aib, newspec.x, newspec.x+newspec.width,
+                                      newspec.y, newspec.y+newspec.height);
+        if (! ok)
+            ot.error (argv[0], Rib.geterror());
     } else if (newspec.x != Aspec.x || newspec.y != Aspec.y) {
         // only offset changed; don't copy the image or crop, simply
         // adjust the origins.
@@ -1234,9 +1463,21 @@ action_croptofull (int argc, const char *argv[])
     const ImageSpec &Aspec (*A->spec(0,0));
     // Implement by calling action_crop with a geometry specifier built
     // from the current full image size.
+#if 0
+    // This should work...
     std::string size = Strutil::format ("%dx%d%+d%+d",
                                         Aspec.full_width, Aspec.full_height,
                                         Aspec.full_x, Aspec.full_y);
+    // ... but tinyformat doesn't print the sign for '0' values!  It
+    // appears to be a bug with iostream use of 'showpos' format flag,
+    // specific to certain gcc libs, perhaps only on OSX.  Workaround:
+#else
+    std::string size = Strutil::format ("%dx%d%c%d%c%d",
+                                        Aspec.full_width, Aspec.full_height,
+                                        Aspec.full_x >= 0 ? '+' : '-',
+                                        Aspec.full_y >= 0 ? '+' : '-',
+                                        abs(Aspec.full_x), abs(Aspec.full_y));
+#endif
     const char *newargv[2] = { "crop", size.c_str() };
     return action_crop (2, newargv);
 }
@@ -1345,7 +1586,9 @@ action_fixnan (int argc, const char *argv[])
         for (int m = 0;  m < miplevels;  ++m) {
             const ImageBuf &Aib ((*A)(s,m));
             ImageBuf &Rib ((*ot.curimg)(s,m));
-            ImageBufAlgo::fixNonFinite (Rib, Aib, mode);
+            bool ok = ImageBufAlgo::fixNonFinite (Rib, Aib, mode);
+            if (! ok)
+                ot.error (argv[0], Rib.geterror());
         }
     }
              
@@ -1354,14 +1597,251 @@ action_fixnan (int argc, const char *argv[])
 
 
 
+static int
+action_over (int argc, const char *argv[])
+{
+    if (ot.postpone_callback (2, action_over, argc, argv))
+        return 0;
+
+    ImageRecRef B (ot.pop());
+    ImageRecRef A (ot.pop());
+    ot.read (A);
+    ot.read (B);
+    const ImageBuf &Aib ((*A)());
+    const ImageBuf &Bib ((*B)());
+    const ImageSpec &specA = Aib.spec();
+    const ImageSpec &specB = Bib.spec();
+
+    // Create output image specification.
+    ImageSpec specR = specA;
+    set_roi (specR, roi_union (get_roi(specA), get_roi(specB)));
+    specR.nchannels = std::max (specA.nchannels, specB.nchannels);
+    if (specR.alpha_channel < 0 && specR.nchannels == 4)
+        specR.alpha_channel = 3;
+
+    ot.push (new ImageRec ("irec", specR, ot.imagecache));
+    ImageBuf &Rib ((*ot.curimg)());
+
+    bool ok = ImageBufAlgo::over (Rib, Aib, Bib);
+    if (! ok)
+        ot.error (argv[0], Rib.geterror());
+    return 0;
+}
+
+
+
+static int
+action_fill (int argc, const char *argv[])
+{
+    if (ot.postpone_callback (1, action_fill, argc, argv))
+        return 0;
+
+    // Read and copy the top-of-stack image
+    ImageRecRef A (ot.pop());
+    ot.read (A);
+    ot.push (new ImageRec (*A, 0, 0, true, false));
+    ImageBuf &Rib ((*ot.curimg)(0,0));
+    const ImageSpec &Rspec = Rib.spec();
+    bool ok = ImageBufAlgo::zero (Rib);
+    if (! ok)
+        ot.error (argv[0], Rib.geterror());
+
+    int w = Rib.spec().width, h = Rib.spec().height;
+    int x = Rib.spec().x, y = Rib.spec().y;
+    if (! adjust_geometry (w, h, x, y, argv[1], true)) {
+        return 0;
+    }
+
+    float *color = ALLOCA (float, Rspec.nchannels);
+    for (int c = 0;  c < Rspec.nchannels;  ++c)
+        color[c] = 1.0f;
+
+    // Parse optional arguments for overrides
+    std::string command = argv[0];
+    size_t pos;
+    while ((pos = command.find_first_of(":")) != std::string::npos) {
+        command = command.substr (pos+1, std::string::npos);
+        if (Strutil::istarts_with(command,"color=")) {
+            // Parse comma-separated color list
+            size_t numpos = 6;
+            for (int c = 0; c < Rspec.nchannels && numpos < command.size() && command[numpos] != ':'; ++c) {
+                color[c] = atof (command.c_str()+numpos);
+                while (numpos < command.size() && command[numpos] != ':' && command[numpos] != ',')
+                    ++numpos;
+                if (numpos < command.size())
+                    ++numpos;
+            }
+        }
+    }
+
+    ok = ImageBufAlgo::fill (Rib, color, ROI(x, x+w, y, y+h));
+    if (! ok)
+        ot.error (argv[0], Rib.geterror());
+
+    return 0;
+}
+
+
+
+static int
+action_text (int argc, const char *argv[])
+{
+    if (ot.postpone_callback (1, action_text, argc, argv))
+        return 0;
+
+    // Read and copy the top-of-stack image
+    ImageRecRef A (ot.pop());
+    ot.read (A);
+    ot.push (new ImageRec (*A, 0, 0, true, false));
+    ImageBuf &Rib ((*ot.curimg)(0,0));
+    const ImageSpec &Rspec = Rib.spec();
+    bool ok = ImageBufAlgo::zero (Rib);
+    if (! ok)
+        ot.error (argv[0], Rib.geterror());
+
+    // Set up defaults for text placement, size, font, color
+    int x = Rspec.x + Rspec.width/2;
+    int y = Rspec.y + Rspec.height/2;
+    int fontsize = 16;
+    std::string font = "";
+    float *textcolor = ALLOCA (float, Rspec.nchannels);
+    for (int c = 0;  c < Rspec.nchannels;  ++c)
+        textcolor[c] = 1.0f;
+
+    // Parse optional arguments for overrides
+    std::string command = argv[0];
+    size_t pos;
+    while ((pos = command.find_first_of(":")) != std::string::npos) {
+        command = command.substr (pos+1, std::string::npos);
+        if (Strutil::istarts_with(command,"x=")) {
+            x = atoi (command.c_str()+2);
+        } else if (Strutil::istarts_with(command,"y=")) {
+            y = atoi (command.c_str()+2);
+        } else if (Strutil::istarts_with(command,"size=")) {
+            fontsize = atoi (command.c_str()+5);
+        } else if (Strutil::istarts_with(command,"color=")) {
+            // Parse comma-separated color list
+            size_t numpos = 6;
+            for (int c = 0; c < Rspec.nchannels && numpos < command.size() && command[numpos] != ':'; ++c) {
+                textcolor[c] = atof (command.c_str()+numpos);
+                while (numpos < command.size() && command[numpos] != ':' && command[numpos] != ',')
+                    ++numpos;
+                if (numpos < command.size())
+                    ++numpos;
+            }
+        } else if (Strutil::istarts_with(command,"font=")) {
+            font = "";
+            size_t s = 5;
+            bool quote = (command[s] == '\"');
+            if (quote)
+                ++s;
+            for ( ; s < command.size() && command[s] != ':' && command[s] != '\"'; ++s)
+                font += command[s];
+        }
+    }
+
+    ok = ImageBufAlgo::render_text (Rib, x, y, argv[1] /* the text */,
+                                    fontsize, font, textcolor);
+    if (! ok)
+        ot.error (argv[0], Rib.geterror());
+
+    return 0;
+}
+
+
+
+/// action_histogram ---------------------------------------------------------
+/// Usage:
+///                   ./oiiotool in --histogram:cumulative=int 'bins'x'height'
+///                   channel -o out
+///
+/// in              - Input image that contains the channel to be histogramed.
+/// cumulative      - Optional argument that can take values 0 or 1. If 0,
+///                   then each bin will contain the count of pixels having
+///                   values in the range for that bin. If 1, then each bin
+///                   will contain not only its count, but also the counts of
+///                   all preceding bins.
+/// 'bins'x'height' - Width and height of the histogram, where width equals
+///                   the number of bins.
+/// channel         - The channel in the input image to be histogramed.
+/// out             - Output image.
+///
+/// Examples:
+///                 - ./oiiotool in --histogram 256x256 0 -o out
+///
+///                   Save the non-cumulative histogram of channel 0 in image
+///                   'in', as an image with size 256x256.
+///
+///                 - ./oiiotool in --histogram:cumulative=1 256x256 0 -o out
+///
+///                   Same as the previous example, but now a cumulative
+///                   histogram is created, instead of a regular one.
+/// --------------------------------------------------------------------------
+static int
+action_histogram (int argc, const char *argv[])
+{
+    ASSERT (argc == 3);
+    if (ot.postpone_callback (1, action_histogram, argc, argv))
+        return 0;
+
+    // Input image.
+    ot.read ();
+    ImageRecRef A (ot.pop());
+    const ImageBuf &Aib ((*A)());
+
+    // Get arguments from command line.
+    const char *size = argv[1];
+    int channel = atoi (argv[2]);
+
+    int cumulative = 0;
+    std::string cmd = argv[0];
+    size_t pos;
+    while ((pos = cmd.find_first_of(":")) != std::string::npos) {
+        cmd = cmd.substr (pos+1, std::string::npos);
+        if (Strutil::istarts_with(cmd,"cumulative="))
+            cumulative = atoi(cmd.c_str()+11);
+    }
+
+    // Extract bins and height from size.
+    int bins = 0, height = 0;
+    if (sscanf (size, "%dx%d", &bins, &height) != 2) {
+        std::cerr << "Invalid size" << "\n";
+        return -1;
+    }
+
+    // Compute regular histogram.
+    std::vector<imagesize_t> hist;
+    bool ok = ImageBufAlgo::histogram (Aib, channel, hist, bins);
+    if (! ok) {
+        ot.error (argv[0], Aib.geterror());
+        return 0;
+    }
+
+    // Compute cumulative histogram if specified.
+    if (cumulative == 1)
+        for (int i = 1; i < bins; i++)
+            hist[i] += hist[i-1];
+
+    // Output image.
+    ImageSpec specR (bins, height, 1, TypeDesc::FLOAT);
+    ot.push (new ImageRec ("irec", specR, ot.imagecache));
+    ImageBuf &Rib ((*ot.curimg)());
+
+    ok = ImageBufAlgo::histogram_draw (Rib, hist);
+    if (! ok)
+        ot.error (argv[0], Rib.geterror());
+
+    return 0;
+}
+
+
+
 static void
 getargs (int argc, char *argv[])
 {
     bool help = false;
-    ArgParse ap;
-    bool dummybool;
-    int dummyint;
-    std::string dummystr;
+    ArgParse ap (argc, (const char **)argv);
+    ot.full_command_line = ap.command_line();
     ap.options ("oiiotool -- simple image processing operations\n"
                 OIIO_INTRO_STRING "\n"
                 "Usage:  oiiotool [filename,option,action]...\n",
@@ -1379,9 +1859,9 @@ getargs (int argc, char *argv[])
                 "--noclobber", &ot.noclobber, "", // synonym
                 "--threads %@ %d", set_threads, &ot.threads, "Number of threads (default 0 == #cores)",
                 "<SEPARATOR>", "Commands that write images:",
-                "-o %@ %s", output_file, &dummystr, "Output the current image to the named file",
+                "-o %@ %s", output_file, NULL, "Output the current image to the named file",
                 "<SEPARATOR>", "Options that affect subsequent image output:",
-                "-d %@ %s", set_dataformat, &dummystr,
+                "-d %@ %s", set_dataformat, NULL,
                     "Set the output data format to one of: "
                     "uint8, sint8, uint10, uint12, uint16, sint16, half, float, double",
                 "--scanline", &ot.output_scanline, "Output scanline images",
@@ -1396,19 +1876,19 @@ getargs (int argc, char *argv[])
                 "--noautocrop %!", &ot.output_autocrop, 
                     "Do not automatically crop images whose formats don't support separate pixel data and full/display windows",
                 "<SEPARATOR>", "Options that change current image metadata (but not pixel values):",
-                "--attrib %@ %s %s", set_any_attribute, &dummystr, &dummystr, "Sets metadata attribute (name, value)",
-                "--sattrib %@ %s %s", set_string_attribute, &dummystr, &dummystr, "Sets string metadata attribute (name, value)",
-                "--caption %@ %s", set_caption, &dummystr, "Sets caption (ImageDescription metadata)",
-                "--keyword %@ %s", set_keyword, &dummystr, "Add a keyword",
-                "--clear-keywords %@", clear_keywords, &dummybool, "Clear all keywords",
-                "--orientation %@ %d", set_orientation, &dummyint, "Set the assumed orientation",
-                "--rotcw %@", rotate_orientation, &dummybool, "Rotate orientation 90 deg clockwise",
-                "--rotccw %@", rotate_orientation, &dummybool, "Rotate orientation 90 deg counter-clockwise",
-                "--rot180 %@", rotate_orientation, &dummybool, "Rotate orientation 180 deg",
-                "--origin %@ %s", set_origin, &dummystr,
+                "--attrib %@ %s %s", set_any_attribute, NULL, NULL, "Sets metadata attribute (name, value)",
+                "--sattrib %@ %s %s", set_string_attribute, NULL, NULL, "Sets string metadata attribute (name, value)",
+                "--caption %@ %s", set_caption, NULL, "Sets caption (ImageDescription metadata)",
+                "--keyword %@ %s", set_keyword, NULL, "Add a keyword",
+                "--clear-keywords %@", clear_keywords, NULL, "Clear all keywords",
+                "--orientation %@ %d", set_orientation, NULL, "Set the assumed orientation",
+                "--rotcw %@", rotate_orientation, NULL, "Rotate orientation 90 deg clockwise",
+                "--rotccw %@", rotate_orientation, NULL, "Rotate orientation 90 deg counter-clockwise",
+                "--rot180 %@", rotate_orientation, NULL, "Rotate orientation 180 deg",
+                "--origin %@ %s", set_origin, NULL,
                     "Set the pixel data window origin (e.g. +20+10)",
-                "--fullsize %@ %s", set_fullsize, &dummystr, "Set the display window (e.g., 1920x1080, 1024x768+100+0, -20-30)",
-                "--fullpixels %@", set_full_to_pixels, &dummybool, "Set the 'full' image range to be the pixel data window",
+                "--fullsize %@ %s", set_fullsize, NULL, "Set the display window (e.g., 1920x1080, 1024x768+100+0, -20-30)",
+                "--fullpixels %@", set_full_to_pixels, NULL, "Set the 'full' image range to be the pixel data window",
                 "<SEPARATOR>", "Options that affect subsequent actions:",
                 "--fail %g", &ot.diff_failthresh, "Failure threshold difference (0.000001)",
                 "--failpercent %g", &ot.diff_failpercent, "Allow this percentage of failures in diff (0)",
@@ -1417,28 +1897,38 @@ getargs (int argc, char *argv[])
                 "--warnpercent %g", &ot.diff_warnpercent, "Allow this percentage of warnings in diff (0)",
                 "--hardwarn %g", &ot.diff_hardwarn, "Warn if any one pixel difference exceeds this error (infinity)",
                 "<SEPARATOR>", "Actions:",
-                "--create %@ %s %d", action_create, &dummystr, &dummyint,
+                "--create %@ %s %d", action_create, NULL, NULL,
                         "Create a blank image (args: geom, channels)",
                 "--pattern %@ %s %s %d", action_pattern, NULL, NULL, NULL,
                         "Create a patterned image (args: pattern, geom, channels)",
-                "--unmip %@", action_unmip, &dummybool, "Discard all but the top level of a MIPmap",
-                "--selectmip %@ %d", action_selectmip, &dummyint,
-                    "Select just one MIP level (0 = highest res)",
-                "--subimage %@ %d", action_select_subimage, &dummyint, "Select just one subimage",
-                "--diff %@", action_diff, &dummybool, "Print report on the difference of two images (modified by --fail, --failpercent, --hardfail, --warn, --warnpercent --hardwarn)",
-                "--add %@", action_add, &dummybool, "Add two images",
-                "--sub %@", action_sub, &dummybool, "Subtract two images",
-                "--abs %@", action_abs, &dummybool, "Take the absolute value of the image pixels",
-                "--flip %@", action_flip, &dummybool, "Flip the image vertically (top<->bottom)",
-                "--flop %@", action_flop, &dummybool, "Flop the image horizontally (left<->right)",
-                "--flipflop %@", action_flipflop, &dummybool, "Flip and flop the image (180 degree rotation)",
-                "--crop %@ %s", action_crop, &dummystr, "Set pixel data resolution and offset, cropping or padding if necessary (WxH+X+Y or xmin,ymin,xmax,ymax)",
-                "--croptofull %@", action_croptofull, &dummybool, "Crop or pad to make pixel data region match the \"full\" region",
-                "--resize %@ %s", action_resize, &dummystr, "Resize (640x480, 50%)",
+                "--capture %@", action_capture, NULL,
+                        "Capture an image (args: camera=%%d)",
+                "--diff %@", action_diff, NULL, "Print report on the difference of two images (modified by --fail, --failpercent, --hardfail, --warn, --warnpercent --hardwarn)",
+                "--add %@", action_add, NULL, "Add two images",
+                "--sub %@", action_sub, NULL, "Subtract two images",
+                "--abs %@", action_abs, NULL, "Take the absolute value of the image pixels",
+                "--over %@", action_over, NULL, "'Over' composite of two images",
+                "--histogram %@ %s %d", action_histogram, NULL, NULL, "Histogram one channel (args: cumulative=0)",
+                "--flip %@", action_flip, NULL, "Flip the image vertically (top<->bottom)",
+                "--flop %@", action_flop, NULL, "Flop the image horizontally (left<->right)",
+                "--flipflop %@", action_flipflop, NULL, "Flip and flop the image (180 degree rotation)",
+                "--crop %@ %s", action_crop, NULL, "Set pixel data resolution and offset, cropping or padding if necessary (WxH+X+Y or xmin,ymin,xmax,ymax)",
+                "--croptofull %@", action_croptofull, NULL, "Crop or pad to make pixel data region match the \"full\" region",
+                "--resize %@ %s", action_resize, NULL, "Resize (640x480, 50%)",
                 "--fixnan %@ %s", action_fixnan, NULL, "Fix NaN/Inf values in the image (options: none, black, box3)",
-                "--pop %@", action_pop, &dummybool,
+                "--fill %@ %s", action_fill, NULL, "Fill a region (options: x=, y=, size=, color=)",
+                "--text %@ %s", action_text, NULL,
+                    "Render text into the current image (options: x=, y=, size=, color=)",
+                "<SEPARATOR>", "Image stack manipulation:",
+                "--ch %@ %s", action_channels, NULL,
+                    "Select or shuffle channels (e.g., \"R,G,B\", \"B,G,R\", \"2,3,4\")",
+                "--unmip %@", action_unmip, NULL, "Discard all but the top level of a MIPmap",
+                "--selectmip %@ %d", action_selectmip, NULL,
+                    "Select just one MIP level (0 = highest res)",
+                "--subimage %@ %d", action_select_subimage, NULL, "Select just one subimage",
+                "--pop %@", action_pop, NULL,
                     "Throw away the current image",
-                "--dup %@", action_dup, &dummybool,
+                "--dup %@", action_dup, NULL,
                     "Duplicate the current image (push a copy onto the stack)",
                 "<SEPARATOR>", "Color management:",
                 "--iscolorspace %@ %s", set_colorspace, NULL,
