@@ -36,6 +36,7 @@
 #include "dassert.h"
 #include "typedesc.h"
 #include "imageio.h"
+#include "filesystem.h"
 #include "fmath.h"
 
 #include "rla_pvt.h"
@@ -150,11 +151,11 @@ private:
 // Obligatory material to make this a recognizeable imageio plugin:
 OIIO_PLUGIN_EXPORTS_BEGIN
 
-DLLEXPORT ImageInput *rla_input_imageio_create () { return new RLAInput; }
+OIIO_EXPORT ImageInput *rla_input_imageio_create () { return new RLAInput; }
 
-DLLEXPORT int rla_imageio_version = OIIO_PLUGIN_VERSION;
+OIIO_EXPORT int rla_imageio_version = OIIO_PLUGIN_VERSION;
 
-DLLEXPORT const char * rla_input_extensions[] = {
+OIIO_EXPORT const char * rla_input_extensions[] = {
     "rla", NULL
 };
 
@@ -167,7 +168,7 @@ RLAInput::open (const std::string &name, ImageSpec &newspec)
 {
     m_filename = name;
 
-    m_file = fopen (name.c_str(), "rb");
+    m_file = Filesystem::fopen (name, "rb");
     if (! m_file) {
         error ("Could not open file \"%s\"", name.c_str());
         return false;
