@@ -72,6 +72,9 @@ private:
         m_file = NULL;
     }
 
+    // Helper function to write the TGA 2.0 data fields, called by close()
+    bool write_tga20_data_fields ();
+
     /// Helper function to flush a non-run-length packet
     ///
     inline void flush_rawp (unsigned char *& src, int size, int start);
@@ -79,6 +82,44 @@ private:
     /// Helper function to flush a run-length packet
     ///
     inline void flush_rlp (unsigned char *buf, int size);
+
+    /// Helper - write, with error detection (no byte swapping!)
+    template <class T>
+    bool fwrite (const T *buf, size_t itemsize=sizeof(T), size_t nitems=1) {
+        if (itemsize*nitems == 0)
+            return true;
+        size_t n = std::fwrite (buf, itemsize, nitems, m_file);
+        if (n != nitems)
+            error ("Write error: wrote %d records of %d", (int)n, (int)nitems);
+        return n == nitems;
+    }
+
+    /// Helper -- write a 'short' with byte swapping if necessary
+    bool fwrite (uint16_t s) {
+        if (bigendian())
+            swap_endian (&s);
+        return fwrite (&s, sizeof(s), 1);
+    }
+    bool fwrite (uint32_t i) {
+        if (bigendian())
+            swap_endian (&i);
+        return fwrite (&i, sizeof(i), 1);
+    }
+
+    /// Helper -- pad with zeroes
+    bool pad (size_t n=1) {
+        while (n--)
+            if (fputc (0, m_file))
+                return false;
+        return true;
+    }
+
+    /// Helper -- write string, with padding and/or truncation
+    bool fwrite_padded (const std::string &s, size_t len) {
+        size_t slen = std::min (s.length(), len-1);
+        return fwrite (s.c_str(), slen)  &&  pad(len-slen);
+    }
+        
 };
 
 
@@ -87,11 +128,11 @@ private:
 // Obligatory material to make this a recognizeable imageio plugin:
 OIIO_PLUGIN_EXPORTS_BEGIN
 
-DLLEXPORT ImageOutput *targa_output_imageio_create () { return new TGAOutput; }
+OIIO_EXPORT ImageOutput *targa_output_imageio_create () { return new TGAOutput; }
 
-// DLLEXPORT int tga_imageio_version = OIIO_PLUGIN_VERSION;   // it's in tgainput.cpp
+// OIIO_EXPORT int tga_imageio_version = OIIO_PLUGIN_VERSION;   // it's in tgainput.cpp
 
-DLLEXPORT const char * targa_output_extensions[] = {
+OIIO_EXPORT const char * targa_output_extensions[] = {
     "tga", "tpic", NULL
 };
 
@@ -125,12 +166,6 @@ TGAOutput::open (const std::string &name, const ImageSpec &userspec,
     close ();  // Close any already-opened file
     m_spec = userspec;  // Stash the spec
 
-    m_file = fopen (name.c_str(), "wb");
-    if (! m_file) {
-        error ("Could not open file \"%s\"", name.c_str());
-        return false;
-    }
-
     // Check for things this format doesn't support
     if (m_spec.width < 1 || m_spec.height < 1) {
         error ("Image resolution must be at least 1x1, you asked for %d x %d",
@@ -150,6 +185,12 @@ TGAOutput::open (const std::string &name, const ImageSpec &userspec,
         return false;
     }
 
+    m_file = Filesystem::fopen (name, "wb");
+    if (! m_file) {
+        error ("Could not open file \"%s\"", name.c_str());
+        return false;
+    }
+
     // Force 8 bit integers
     m_spec.set_format (TypeDesc::UINT8);
 
@@ -205,24 +246,30 @@ TGAOutput::open (const std::string &name, const ImageSpec &userspec,
     // due to struct packing, we may get a corrupt header if we just dump the
     // struct to the file; to adress that, write every member individually
     // save some typing
-#define WH(memb)    fwrite (&tga.memb, sizeof (tga.memb), 1, m_file)
-    WH(idlen);
-    WH(cmap_type);
-    WH(type);
-    WH(cmap_first);
-    WH(cmap_length);
-    WH(cmap_size);
-    WH(x_origin);
-    WH(y_origin);
-    WH(width);
-    WH(height);
-    WH(bpp);
-    WH(attr);
-#undef WH
+    if (!fwrite(&tga.idlen) ||
+        !fwrite(&tga.cmap_type) ||
+        !fwrite(&tga.type) ||
+        !fwrite(&tga.cmap_first) ||
+        !fwrite(&tga.cmap_length) ||
+        !fwrite(&tga.cmap_size) ||
+        !fwrite(&tga.x_origin) ||
+        !fwrite(&tga.y_origin) ||
+        !fwrite(&tga.width) ||
+        !fwrite(&tga.height) ||
+        !fwrite(&tga.bpp) ||
+        !fwrite(&tga.attr)) {
+        fclose (m_file);  m_file = NULL;
+        return false;
+    }
+
 
     // dump comment to file, don't bother about null termination
-    if (tga.idlen)
-        fwrite (id.c_str(), tga.idlen, 1, m_file);
+    if (tga.idlen) {
+        if (!fwrite(id.c_str(), tga.idlen)) {
+            fclose (m_file);  m_file = NULL;
+            return false;
+        }
+    }
 
     return true;
 }
@@ -230,7 +277,7 @@ TGAOutput::open (const std::string &name, const ImageSpec &userspec,
 
 
 bool
-TGAOutput::close ()
+TGAOutput::write_tga20_data_fields ()
 {
     if (m_file) {
         // write out the TGA 2.0 data fields
@@ -242,238 +289,155 @@ TGAOutput::close ()
         fseek (m_file, 0, SEEK_END);
 
         // write out the thumbnail, if there is one
-        int ofs_thumb = 0;
-        {
-            unsigned char tw = m_spec.get_int_attribute ("thumbnail_width", 0);
-            if (tw) {
-                unsigned char th = m_spec.get_int_attribute ("thumbnail_width",
-                                                             0);
-                if (th) {
-                    int tc = m_spec.get_int_attribute ("thumbnail_nchannels",
-                                                       0);
-                    if (tc == m_spec.nchannels) {
-                        ImageIOParameter *p =
-                            m_spec.find_attribute ("thumbnail_image");
-                        if (p) {
-                            ofs_thumb = ftell (m_file);
-                            if (bigendian())
-                                swap_endian (&ofs_thumb);
-                            // dump thumbnail size
-                            fwrite (&tw, 1, 1, m_file);
-                            fwrite (&th, 1, 1, m_file);
-                            // dump thumbnail data
-                            fwrite (p->data(), p->datasize(), 1, m_file);
-                        }
-                    }
+        uint32_t ofs_thumb = 0;
+        unsigned char tw = m_spec.get_int_attribute ("thumbnail_width", 0);
+        unsigned char th = m_spec.get_int_attribute ("thumbnail_width", 0);
+        int tc = m_spec.get_int_attribute ("thumbnail_nchannels", 0);
+        if (tw && th && tc == m_spec.nchannels) {
+            ImageIOParameter *p = m_spec.find_attribute ("thumbnail_image");
+            if (p) {
+                ofs_thumb = (uint32_t) ftell (m_file);
+                // dump thumbnail size
+                if (!fwrite (&tw) ||
+                    !fwrite (&th) ||
+                    !fwrite (p->data(), p->datasize())) {
+                    return false;
                 }
             }
         }
         
         // prepare the footer
         tga_footer foot = {(uint32_t)ftell (m_file), 0, "TRUEVISION-XFILE."};
-        if (bigendian()) {
-            swap_endian (&foot.ofs_ext);
-            swap_endian (&foot.ofs_dev);
-        }
 
         // write out the extension area
-        // ext area size
-        short tmpint = 495;
-        if (bigendian())
-            swap_endian (&tmpint);
-        fwrite (&tmpint, sizeof (tmpint), 1, m_file);
 
-        tmpint = 0;
+        // ext area size -- 2 bytes, always short(495)
+        fwrite (uint16_t(495));
 
-        // author
-        std::string tmpstr = m_spec.get_string_attribute ("Artist", "");
-        fwrite (tmpstr.c_str(), std::min (tmpstr.length (), size_t(40)),
-                1, m_file);
-        // fill the rest with zeros
-        for (int i = 41 - std::min (tmpstr.length (), size_t(40)); i > 0; i--)
-            fwrite (&tmpint, 1, 1, m_file);
+        // author - 41 bytes
+        fwrite_padded (m_spec.get_string_attribute("Artist"), 41);
 
-        // image comment
-        tmpstr = m_spec.get_string_attribute ("ImageDescription", "");
+        // image comment - 324 bytes
+        fwrite_padded (m_spec.get_string_attribute("ImageDescription"), 324);
+
+        // timestamp - 6 shorts (month, day, year, hour, minute, second)
         {
-            char *p = (char *)tmpstr.c_str ();
-            int w = 0;  // number of bytes written
-            for (int pos = 0; w < 324 && pos < (int)tmpstr.length ();
-                 w++, pos++) {
-                // on line breaks, fill the remainder of the line with zeros
-                if (p[pos] == '\n') {
-                    while ((w + 1) % 81 != 0) {
-                        fwrite (&tmpint, 1, 1, m_file);
-                        w++;
-                    }
-                    continue;
-                }
-                fwrite (&p[pos], 1, 1, m_file);
-                // null-terminate each line
-                if ((w + 1) % 81 == 0) {
-                    fwrite (&tmpint, 1, 1, m_file);
-                    w++;
-                }
-            }
-            // fill the rest with zeros
-            for (; w < 324; w++)
-                fwrite (&tmpint, 1, 1, m_file);
-        }
-
-        // timestamp
-        tmpstr = m_spec.get_string_attribute ("DateTime", "");
-        {
-            unsigned short y, m, d, h, i, s;
-            if (tmpstr.length () > 0)
-                sscanf (tmpstr.c_str (), "%04hu:%02hu:%02hu %02hu:%02hu:%02hu",
+            std::string dt = m_spec.get_string_attribute ("DateTime", "");
+            uint16_t y, m, d, h, i, s;
+            if (dt.length() > 0)
+                sscanf (dt.c_str(), "%04hu:%02hu:%02hu %02hu:%02hu:%02hu",
                         &y, &m, &d, &h, &i, &s);
             else
                 y = m = d = h = i = s = 0;
-            if (bigendian()) {
-                swap_endian (&y);
-                swap_endian (&m);
-                swap_endian (&d);
-                swap_endian (&h);
-                swap_endian (&i);
-                swap_endian (&s);
+            if (!fwrite(m) || !fwrite(d) || !fwrite(y) ||
+                !fwrite(h) || !fwrite(i) || !fwrite(s)) {
+                return false;
             }
-            fwrite (&m, sizeof (m), 1, m_file);
-            fwrite (&d, sizeof (d), 1, m_file);
-            fwrite (&y, sizeof (y), 1, m_file);
-            fwrite (&h, sizeof (h), 1, m_file);
-            fwrite (&i, sizeof (i), 1, m_file);
-            fwrite (&s, sizeof (s), 1, m_file);
         }
 
-        // job ID
-        tmpstr = m_spec.get_string_attribute ("DocumentName", "");
-        fwrite (tmpstr.c_str(), std::min (tmpstr.length (), size_t(40)),
-                1, m_file);
-        // fill the rest with zeros
-        for (int i = 41 - std::min (tmpstr.length (), size_t(40)); i > 0; i--)
-            fwrite (&tmpint, 1, 1, m_file);
+        // job ID - 41 bytes
+        fwrite_padded (m_spec.get_string_attribute("DocumentName"), 41);
 
-        // job time
-        tmpstr = m_spec.get_string_attribute ("targa:JobTime", "");
+        // job time - 3 shorts (hours, minutes, seconds)
         {
-            unsigned short h, m, s;
-            if (tmpstr.length () > 0)
-                sscanf (tmpstr.c_str (), "%hu:%02hu:%02hu", &h, &m, &s);
+            std::string jt = m_spec.get_string_attribute ("targa:JobTime", "");
+            uint16_t h, m, s;
+            if (jt.length() > 0)
+                sscanf (jt.c_str(), "%hu:%02hu:%02hu", &h, &m, &s);
             else
                 h = m = s = 0;
-            if (bigendian()) {
-                swap_endian (&h);
-                swap_endian (&m);
-                swap_endian (&s);
-            }
-            fwrite (&h, sizeof (h), 1, m_file);
-            fwrite (&m, sizeof (m), 1, m_file);
-            fwrite (&s, sizeof (s), 1, m_file);
+            if (!fwrite(h) || !fwrite(m) || !fwrite(s))
+                return false;
         }
 
-        // software ID - we advertise ourselves
-        tmpstr = OIIO_INTRO_STRING;
-        fwrite (tmpstr.c_str(), std::min (tmpstr.length (), size_t(40)),
-                1, m_file);
-        // fill the rest with zeros
-        for (int i = 41 - std::min (tmpstr.length (), size_t(40)); i > 0; i--)
-            fwrite (&tmpint, 1, 1, m_file);
+        // software ID -- 41 bytes
+        fwrite_padded (m_spec.get_string_attribute("Software"), 41);
 
-        // software version
-        {
-            short v = OIIO_VERSION_MAJOR * 100
-                    + OIIO_VERSION_MINOR * 10
-                    + OIIO_VERSION_PATCH;
-            if (bigendian())
-                swap_endian (&v);
-            fwrite (&v, sizeof (v), 1, m_file);
-            fwrite (&tmpint, 1, 1, m_file);
-        }
+        // software version - 3 bytes (first 2 bytes: version*100)
+        if (!fwrite(uint16_t(OIIO_VERSION)))
+            return false;
+        pad (1);
 
-        // key colour
-        // FIXME: what do we save here?
-        fwrite (&tmpint, 2, 1, m_file);
-        fwrite (&tmpint, 2, 1, m_file);
+        // key colour (ARGB) -- punt and write 0
+        pad (4);
 
-        // pixel aspect ratio
+        // pixel aspect ratio -- two shorts, giving a ratio
         {
             float ratio = m_spec.get_float_attribute ("PixelAspectRatio", 1.f);
             float EPS = 1E-5f;
             if (ratio >= (0.f+EPS) && ((ratio <= (1.f-EPS))||(ratio >= (1.f+EPS)))) {
                 // FIXME: invent a smarter way to convert to a vulgar fraction?
                 // numerator
-                tmpint = (unsigned short)(ratio * 10000.f);
-                fwrite (&tmpint, 2, 1, m_file);
-                // denominator
-                tmpint = 10000;
-                fwrite (&tmpint, 2, 1, m_file);
-                // reset tmpint value
-                tmpint = 0;
+                fwrite (uint16_t(ratio * 10000.f));  // numerator
+                fwrite (uint16_t(10000));            // denominator
             } else {
                 // just dump two zeros in there
-                fwrite (&tmpint, 2, 1, m_file);
-                fwrite (&tmpint, 2, 1, m_file);
+                fwrite (uint16_t(0));
+                fwrite (uint16_t(0));
             }
         }
 
-        // gamma
-        {
-            if (Strutil::iequals (m_spec.get_string_attribute ("oiio:ColorSpace"), "GammaCorrected")) {
-                float gamma = m_spec.get_float_attribute ("oiio:Gamma", 1.0);
-                
-                // FIXME: invent a smarter way to convert to a vulgar fraction?
-                // NOTE: the spec states that only 1 decimal place of precision
-                // is needed, thus the expansion by 10
-                // numerator
-                tmpint = (unsigned short)(gamma * 10.f);
-                fwrite (&tmpint, 2, 1, m_file);
-                // denominator
-                tmpint = 10;
-                fwrite (&tmpint, 2, 1, m_file);
-                // reset tmpint value
-                tmpint = 0;
-            } else {
-                // just dump two zeros in there
-                fwrite (&tmpint, 2, 1, m_file);
-                fwrite (&tmpint, 2, 1, m_file);
-            }
+        // gamma -- two shorts, giving a ratio
+        if (Strutil::iequals (m_spec.get_string_attribute("oiio:ColorSpace"), "GammaCorrected")) {
+            float gamma = m_spec.get_float_attribute ("oiio:Gamma", 1.0);
+            // FIXME: invent a smarter way to convert to a vulgar fraction?
+            // NOTE: the spec states that only 1 decimal place of precision
+            // is needed, thus the expansion by 10
+            // numerator
+            fwrite (uint16_t(gamma*10.0f));
+            fwrite (uint16_t(10));
+        } else {
+            // just dump two zeros in there
+            fwrite (uint16_t(0));
+            fwrite (uint16_t(0));
         }
 
-        // offset to colour correction table
+        // offset to colour correction table - 4 bytes
         // FIXME: support this once it becomes clear how it's actually supposed
         // to be used... the spec is very unclear about this
         // for the time being just dump four NULL bytes
-        fwrite (&tmpint, 2, 1, m_file);
-        fwrite (&tmpint, 2, 1, m_file);
+        pad (4);
 
-        // offset to thumbnail (endiannes has already been accounted for)
-        fwrite (&ofs_thumb, 4, 1, m_file);
+        // offset to thumbnail - 4 bytes
+        if (!fwrite(ofs_thumb))
+            return false;
 
-        // offset to scanline table
+        // offset to scanline table - 4 bytes
         // not used very widely, don't bother unless someone complains
-        fwrite (&tmpint, 2, 1, m_file);
-        fwrite (&tmpint, 2, 1, m_file);
+        pad (4);
 
-        // alpha type
-        {
-            unsigned char at = (m_spec.nchannels % 2 == 0)
+        // alpha type - one byte
+        unsigned char at = (m_spec.nchannels % 2 == 0)
                              ? TGA_ALPHA_USEFUL : TGA_ALPHA_NONE;
-            fwrite (&at, 1, 1, m_file);
-        }
+        if (!fwrite(&at))
+            return false;
 
         // write out the TGA footer
-        fwrite (&foot.ofs_ext, 1, sizeof (foot.ofs_ext), m_file);
-        fwrite (&foot.ofs_dev, 1, sizeof (foot.ofs_dev), m_file);
-        fwrite (&foot.signature, 1, sizeof (foot.signature), m_file);
+        if (!fwrite(foot.ofs_ext) ||
+            !fwrite(foot.ofs_dev) ||
+            !fwrite(&foot.signature, 1, sizeof(foot.signature))) {
+            return false;
+        }
+    }
 
+    return true;
+}
+
+
+
+bool
+TGAOutput::close ()
+{
+    bool ok = true;
+    if (m_file) {
+        ok &= write_tga20_data_fields ();
         // close the stream
         fclose (m_file);
         m_file = NULL;
     }
 
     init ();      // re-initialize
-    return true;  // How can we fail?
-                  // Epicly. -- IneQuation
+    return ok;
 }
 
 
@@ -486,9 +450,11 @@ TGAOutput::flush_rlp (unsigned char *buf, int size)
         return;
     // write packet header
     unsigned char h = (size - 1) | 0x80;
-    fwrite (&h, 1, 1, m_file);
     // write packet pixel
-    fwrite (buf, m_spec.nchannels, 1, m_file);
+    if (!fwrite(&h) || !fwrite (buf, m_spec.nchannels)) {
+        // do something intelligent?
+        return;
+    }
 }
 
 
@@ -501,14 +467,17 @@ TGAOutput::flush_rawp (unsigned char *& src, int size, int start)
         return;
     // write packet header
     unsigned char h = (size - 1) & ~0x80;
-    fwrite (&h, 1, 1, m_file);
+    if (!fwrite (&h))
+        return;
     // rewind the scanline and flush packet pixels
     unsigned char buf[4];
     int n = m_spec.nchannels;
     for (int i = 0; i < size; i++) {
         if (n <= 2) {
             // 1- and 2-channels can write directly
-            fwrite (src+start, 1, n, m_file);
+            if (!fwrite (src+start, n)) {
+                return;
+            }
         } else {
             // 3- and 4-channel must swap red and blue
             buf[0] = src[(start + i) * n + 2];
@@ -516,7 +485,9 @@ TGAOutput::flush_rawp (unsigned char *& src, int size, int start)
             buf[2] = src[(start + i) * n + 0];
             if (n > 3)
                 buf[3] = src[(start + i) * n + 3];
-            fwrite (buf, 1, n, m_file);
+            if (!fwrite (buf, n)) {
+                return;
+            }
         }
     }
 }
@@ -530,7 +501,7 @@ TGAOutput::write_scanline (int y, int z, TypeDesc format,
     y -= m_spec.y;
     m_spec.auto_stride (xstride, format, spec().nchannels);
     data = to_native_scanline (format, data, xstride, m_scratch);
-    if (data != &m_scratch[0]) {
+    if (m_scratch.empty() || data != &m_scratch[0]) {
         m_scratch.assign ((unsigned char *)data,
                           (unsigned char *)data+m_spec.scanline_bytes());
         data = &m_scratch[0];
@@ -660,14 +631,19 @@ TGAOutput::write_scanline (int y, int z, TypeDesc format,
         fseek(m_file, 18 + m_idlen + (m_spec.height - y - 1) * w * n, SEEK_SET);
         if (n <= 2) {
             // 1- and 2-channels can write directly
-            fwrite (bdata, n, w, m_file);
+            if (!fwrite (bdata, n, w)) {
+                return false;
+            }
         } else {
             // 3- and 4-channels must swap R and B
             std::vector<unsigned char> buf;
             buf.assign (bdata, bdata + n*w);
             for (int x = 0; x < m_spec.width; x++)
                 std::swap (buf[x*n], buf[x*n+2]);
-            fwrite (&buf[0], n, w, m_file);
+
+            if (!fwrite (&buf[0], n, w)) {
+                return false;
+            }
         }
     }
 
